# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.slacktokenanalyzer import SlackTokenAnalyzer


class TestSlackTokenAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = SlackTokenAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # slack key dump
        self.paste.body = "xoxb-999999999999-999999999999-999999999999-9999999999999999999999999999999a"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack key dump
        self.paste.body = "xoxb-999999999999-999999999999-999999999999-99999999999999999999999999999999"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack key dump
        self.paste.body = "xoxp-999999999999-999999999999-999999999999-99999999999999999999999999999999"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack key dump
        self.paste.body = "xoxa-999999999999-999999999999-999999999999-99999999999999999999999999999999"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "my token is: xoxo-999999999999-999999999999-999999999999-99999999999999abc999999999999999"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("xoxo-999999999999-999999999999-999999999999-99999999999999abc999999999999999", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "my token is: xoxo-999999999999-999999999999-999999999999-99999999999999abc999999999999999. Please also" \
                          "take xoxa-999999999999-999999999999-999999999999-99999999999999999999999999999999!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("xoxo-999999999999-999999999999-999999999999-99999999999999abc999999999999999", match[0])
        self.assertEqual("xoxa-999999999999-999999999999-999999999999-99999999999999999999999999999999", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = "my bike isn't a number"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "xoxa-999999999999-999999999999-99999999999-99999999999999999999999999999999"
        self.assertFalse(self.analyzer.match(self.paste))

        # Upper-case
        self.paste.body = "my token is: xoxo-999999999999-999999999999-999999999999-99999999999999Abc999999999999999"
        self.assertFalse(self.analyzer.match(self.paste))
        # too short and Upper-case
        self.paste.body = "my token is: xoxo-999999999999-999999999999-99999999999-99999999999999Abc999999999999999"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import logging
import pathlib
import sqlite3

from .abstractdb import AbstractDB


class SQLiteDB(AbstractDB):
    """Database class representing an sqlite database instance"""

    def __init__(self, dbpath="pastepwn"):
        super().__init__()
        self.dbpath = pathlib.Path(dbpath)
        self.logger = logging.getLogger(__name__)
        self.logger.debug("Initializing SQLite - {0}".format(self.dbpath))

        # Check if the folder path exists
        if not self.dbpath.exists():
            # If not, create the path and the file
            dbdir = self.dbpath.parent
            if not dbdir.exists():
                dbdir.mkdir()
            self.dbpath.touch()
        elif self.dbpath.is_dir():
            raise ValueError("'{0}' is a directory. Use different path/name for database.".format(self.dbpath))

        try:
            self.db = sqlite3.connect(str(self.dbpath), check_same_thread=False)
        except Exception as e:
            self.logger.exception("An exception happened when initializing the database: {0}".format(e))
            raise

        self.db.text_factory = lambda x: str(x, "utf-8", "ignore")
        self.cursor = self.db.cursor()
        self._create_tables()

        self.logger.debug("Connected to database!")

    def _create_tables(self):
        self.dbpath.touch()

        self.cursor.execute("""CREATE TABLE IF NOT EXISTS 'pastes' (
                            'key'	TEXT NOT NULL UNIQUE,
                            'title'	TEXT,
                            'user'	TEXT,
                            'size'	INTEGER,
                            'date'	INTEGER,
                            'expire'	INTEGER,
                            'syntax'	TEXT,
                            'scrape_url'	TEXT,
                            'full_url'	TEXT,
                            'body'	TEXT,
                            PRIMARY KEY('key'))""")
        self.db.commit()

    def _insert_data(self, paste):
        self.cursor.execute("INSERT INTO pastes (key, title, user, size, date, expire, syntax, scrape_url, full_url, body) "
                            "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                            (paste.key, paste.title, paste.user, paste.size, paste.date, paste.expire, paste.syntax,
                             paste.scrape_url, paste.full_url, paste.body
                             )
                            )
        self.db.commit()

    def _update_data(self, paste):
        self.cursor.execute("UPDATE pastes SET body = ? WHERE key = ?", (paste.body, paste.key))
        self.db.commit()

    def _get_data(self, key, value):
        pass

    def close_connection(self):
        """Closes the connection to the sqlite database"""
        self.cursor.close()
        self.db.close()

    def count(self, key, value):
        # TODO add filter to counting
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def count_all(self):
        return self.cursor.execute("SELECT count(*) FROM pastes")

    def store(self, paste):
        self.logger.debug("Storing paste {0}".format(paste.key))

        try:
            self._insert_data(paste)
        except Exception as e:
            self.logger.debug("Exception '{0}'".format(e))
            if "UNIQUE constraint failed: pastes.key" in str(e):
                self.logger.debug("Doing upsert")
                self._update_data(paste)

    def get(self, key):
        return self._get_data("key", key)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.phonenumberanalyzer import PhoneNumberAnalyzer


class TestPhoneNumberAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = PhoneNumberAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        self.paste.body = "+14155552671"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "+1 4155552671"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "+49 89-636-48018"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "+357  81-127 29103"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "+1 (811) 1272 103"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "Check out my new number: +1 (811) 1272 103 this is it"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("+1 (811) 1272 103", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "Check out all those numbers: +1 (811) 1272 103\n+357  81-127 29103 and many more"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("+1 (811) 1272 103", match[0])
        self.assertEqual("+357  81-127 29103", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "https://www.google.com"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "123456789"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "123 456 789"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "+42"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "+42 1"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "+123456789012345678901234567890"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.bcrypthashanalyzer import BcryptHashAnalyzer


class TestBcryptHashAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = BcryptHashAnalyzer(None)
        self.paste = mock.Mock()

    def test_match(self):
        valid_hashes = ["$2a$10$BIgnlSmYE8qYiONM0NQ53eRWBw5G4HIJEbXKzcsRVt.08IDnqH/V.",
                        "$2a$11$EppiRqR0kG9EKy56edDWTOnsv/oGW0dqAJB9ucmn3augbmcm8v/iy",
                        "$2b$10$FpOpno43SIE8e1hWnlOdR.9hG2J8dd5FD1kQq8hn4zLdKa5eIiFUO",
                        "$2a$10$SVy7GlMnWsemiZByHSnV0O3WoEHGImFt8v07uH.K3ZXwH5j9o/DP.",
                        "$2a$10$59SNkcZ0rdC2VgeWaavVyea9PFget/xmtbV7.9IeJl3CUq.Q954i2",
                        "$2b$10$Y8CJ9YIwrxt1YYgMqqU1delCYoTpIl18SRtYYI2kyM3jduKPHvWMC",
                        "$2a$10$2tNCUb.FUpSutyhkbqmMBuNnLzhqI4q9Miqurnj6eu.XsiIjww7I6",
                        "$2a$10$OyrADUFmj9QEqsd8frkEDOEYSPQalW5qoI1s2z6taCWwgUsjKzk5m"
                        ]

        invalid_hashes = ["7168D46050573DDA4CE409FA1515638BD28E86346D45F686310ED0678172BABCD4117FD15DD380B964352FE879FB745B573A730D526BB1188B2790FBA06E8ACA",
                          "5FD924625F6AB16A19CC9807C7C506AE1813490E4BA675F843D5A10E0BAACDB8",
                          "522F02FEA11E70C03C90C247C50410443246BFCB",
                          "8433FD5A3B0ED71D21CFB9F291BD89B9",
                          "$2a$124$SVy7GlMnWsemiZByHSnV0O3WoEHGImFt8v07uH.K3ZXwH5j9o/DP.a",
                          "$2a$12$SVy7GlMnWsemiZByHSnV0O3WoEHGImFt8v{07uH.K3ZXwH5j9o/DP.a",
                          "$2a$14$SVy7GlMnWsemiZByHSnV0O3WGImFt8v07uH.K3ZXwH5j9o/DP.a",
                          "@x,Y8q+jnYeZr$;",
                          "This is a test",
                          "$2a$10$   asdf 1234 how are you?"]

        for test_hash in valid_hashes:
            self.paste.body = test_hash
            self.assertTrue(self.analyzer.match(self.paste), test_hash)

        for test_hash in invalid_hashes:
            self.paste.body = test_hash
            self.assertFalse(self.analyzer.match(self.paste), test_hash)

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "We now have a hashe inside the text: $2a$11$EppiRqR0kG9EKy56edDWTOnsv/oGW0dqAJB9ucmn3augbmcm8v/iy and some text here!"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "We now have a hashe inside the text: $2a$11$EppiRqR0kG9EKy56edDWTOnsv/oGW0dqAJB9ucmn3augbmcm8v/iy and some text here!" \
                          "Also there is $2a$10$OyrADUFmj9QEqsd8frkEDOEYSPQalW5qoI1s2z6taCWwgUsjKzk5m as another one"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("$2a$11$EppiRqR0kG9EKy56edDWTOnsv/oGW0dqAJB9ucmn3augbmcm8v/iy", match[0])
        self.assertEqual("$2a$10$OyrADUFmj9QEqsd8frkEDOEYSPQalW5qoI1s2z6taCWwgUsjKzk5m", match[1])

    def test_match_none(self):
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste = None
        self.assertFalse(self.analyzer.match(self.paste))

    def test_match_empty(self):
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

    def test_actions_present(self):
        action = mock.MagicMock(spec=BasicAction)
        analyzer = BcryptHashAnalyzer(action)
        self.assertEqual([action], analyzer.actions)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.awssecretkeyanalyzer import AWSSecretKeyAnalyzer


class TestAWSSecretKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = AWSSecretKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # obtained by pastebin search
        self.paste.body = "HrNMIhjZDnvkH5YGJpwjq0Flmj8H+dvURedLRjsO"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "ZGBMmoyRxdhMObx0EuANS9FiS2kG5FwDVLH2XY7y"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "kYKQ24NoNtmga55G7OVeY/kPAZ7xONl6FfmuXArc"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "9cf95dacd226dcf43da376cdb6cbba7035218921"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "my super cool hash is 9cf95dacd226dcf43da376cdb6cbba7035218921 and here's some more text"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("9cf95dacd226dcf43da376cdb6cbba7035218921", match[0])

        self.paste.body = "AWS Secret Access Key [None]: HrNMIhjZDnvkH5YGJpwjq0Flmj8H+dvURedLRjsO"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("HrNMIhjZDnvkH5YGJpwjq0Flmj8H+dvURedLRjsO", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        # Newline-separated valid hashes
        self.paste.body = "ZGBMmoyRxdhMObx0EuANS9FiS2kG5FwDVLH2XY7y\nHrNMIhjZDnvkH5YGJpwjq0Flmj8H+dvURedLRjsO"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("ZGBMmoyRxdhMObx0EuANS9FiS2kG5FwDVLH2XY7y", match[0])
        self.assertEqual("HrNMIhjZDnvkH5YGJpwjq0Flmj8H+dvURedLRjsO", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid character '-'
        self.paste.body = "9cf95dacd226dcf43da376cdb6cbb-7035218921"
        self.assertFalse(self.analyzer.match(self.paste))

        # MD5 Hash
        self.paste.body = "9e107d9d372bb6826bd81d3542a419d6"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "ZGBMmoyRxdhMObx0EuANS9FiS2kG5FwDVLH2XY7y1"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "ZGBMmoyRxdhMObx0EuANS9FiS2kG5FwDVLH2XY7"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import re
from .basicanalyzer import BasicAnalyzer
from pastepwn.util import listify


class ExactWordAnalyzer(BasicAnalyzer):
    """Analyzer to match the content of a paste by words"""
    name = "ExactWordAnalyzer"

    def __init__(self, actions, words, blacklist=None, case_sensitive=False):
        super().__init__(actions, "{0} ({1})".format(self.name, words))

        self.words = listify(words)
        self.blacklist = blacklist or []
        self.case_sensitive = case_sensitive

    def _blacklist_word_found(self, text):
        blacklist = self.blacklist

        if not len(blacklist):
            return False

        if not self.case_sensitive:
            text = text.lower()
            blacklist = [word.lower() for word in blacklist]

        return any((self._word_in_text(word, text) for word in blacklist))

    def add_word(self, word):
        """
        Add a word to the analyzer
        :param word: Word to be added
        :return:
        """
        self.words.append(word)

    def match(self, paste):
        """Check if the specified words are part of the paste text"""
        if paste is None:
            return False

        paste_content = paste.body or ""

        if self._blacklist_word_found(paste_content):
            return False

        words = self.words

        if not self.case_sensitive:
            paste_content = paste_content.lower()
            words = [word.lower() for word in words]

        return [word for word in words if self._word_in_text(word, paste_content)]

    def _word_in_text(self, word, text):
        pattern = r"\b{0}\b".format(word)
        return re.search(pattern, text) is not None



# -*- coding: utf-8 -*-
import logging
from threading import Thread, current_thread


def start_thread(target, name, exception_event, *args, **kwargs):
    """
    Starts a thread passed as argument and catches exceptions that happens during execution
    :param target: Method to be executed in the thread
    :param name: Name of the thread
    :param exception_event: An event that will be set if an exception occurred
    :param args: Arguments to be passed to the threaded method
    :param kwargs: Keyword-Arguments to be passed to the threaded method
    :return:
    """
    thread = Thread(target=thread_wrapper, name=name, args=(target, exception_event) + args, kwargs=kwargs)
    thread.start()
    return thread


def thread_wrapper(target, exception_event, *args, **kwargs):
    """
    Wrapper around the execution of a passed method, that catches and logs exceptions
    :param target: Method to be executed
    :param exception_event: An event that will be set if an exception occurred
    :param args: Arguments to be passed to the target method
    :param kwargs: Keyword-Arguments to be passed to the target method
    :return:
    """
    thread_name = current_thread().name
    logger = logging.getLogger(__name__)
    logger.debug("{0} - thread started".format(thread_name))
    try:
        target(*args, **kwargs)
    except Exception:
        exception_event.set()
        logger.exception("unhandled exception in %s", thread_name)
        raise
    logger.debug("{0} - thread ended".format(thread_name))


def join_threads(threads):
    """
    End all threads and join them back into the main thread
    :param threads: List of threads to be joined
    :return:
    """
    logger = logging.getLogger(__name__)
    for thread in threads:
        logger.debug("Joining thread {0}".format(thread.name))
        thread.join()
        logger.debug("Thread {0} has ended".format(thread.name))



# -*- coding: utf-8 -*-
import logging

import twitter

from pastepwn.util import TemplatingEngine
from .basicaction import BasicAction


class TwitterAction(BasicAction):
    """Action to tweet a message to a given account"""

    name = "TwitterAction"

    def __init__(
            self,
            consumer_key=None,
            consumer_secret=None,
            access_token_key=None,
            access_token_secret=None,
            template=None,
            ):
        super().__init__()

        self.logger = logging.getLogger(__name__)

        self.twitter_api = twitter.Api(
            consumer_key=consumer_key,
            consumer_secret=consumer_secret,
            access_token_key=access_token_key,
            access_token_secret=access_token_secret,
            )
        self.template = template

    def perform(self, paste, analyzer_name=None, matches=None):
        """Tweet a message"""
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)
        self.twitter_api.PostUpdate(text)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.slackwebhookanalyzer import SlackWebhookAnalyzer


class TestSlackWebhookAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = SlackWebhookAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # slack webhook url (sample)
        self.paste.body = "https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack webhook url (manually generated)
        self.paste.body = "https://hooks.slack.com/services/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"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack webhook url (randomly generated)
        self.paste.body = "https://hooks.slack.com/services/TwLj3Aeic/B2RnzBQQp/7JkqKP9XxuqN3WFDn3tUA8NJ"
        self.assertTrue(self.analyzer.match(self.paste))

        # slack webhook url (randomly generated)
        self.paste.body = "https://hooks.slack.com/services/TafdGEj9a/B9BdR2SLM/yJAk3gcguM8YzFEpaPnSvZ4Q"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "here is the webhook url: The slack webhook key is " \
                          "https://hooks.slack.com/services/T00000000/B00000000" \
                          "/XXXXXXXXXXXXXXXXXXXXXXXX! how about that!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "here is the webhook url: The slack webhook key is " \
                          "https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX! how about that! and now" \
                          "there is another one https://hooks.slack.com/services/T00000000/B00000000/YYYYYYYYYYYYYYYYYYYYYYYY right here!" \
                          "And an invalid url: https://hooks.slack.com/services/T00000000/B00000000/ZZZZZZZZZZ there!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX", match[0])
        self.assertEqual("https://hooks.slack.com/services/T00000000/B00000000/YYYYYYYYYYYYYYYYYYYYYYYY", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Other Slack URL (api docs)
        self.paste.body = "https://api.slack.com/incoming-webhooks"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid Character
        self.paste.body = "https://hooks.slack.com/services/T00!00000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid Length
        self.paste.body = "https://hooks.slack.com/services/T00000000/B0000000/XXXXXXXXXXXXXXXXXXXXXXXX"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid Format (/services/Z... vs /services/T...)
        self.paste.body = "https://hooks.slack.com/services/Z00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.awsaccesskeyanalyzer import AWSAccessKeyAnalyzer


class TestAWSAccessKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = AWSAccessKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # obtained by pastebin search
        self.paste.body = "AKIAIX2GUZJMJFDZON4A"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "AKIAJ5OVIVTO7XQ7UWOQ"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "AKIAJM4DOPAAJWLUJ2PQ"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by pastebin search
        self.paste.body = "AKIAIKSA47YZNJAY2H6A"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_intext(self):
        self.paste.body = "my super cool hash is AKIAJM4DOPAAJWLUJ2PQ and here's some more text"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("AKIAJM4DOPAAJWLUJ2PQ", match[0])

        self.paste.body = "AWS Access Key ID [None]: AKIAIX2GUZJMJFDZON4A"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("AKIAIX2GUZJMJFDZON4A", match[0])

    def test_multiple(self):
        # Newline-separated valid hashes
        self.paste.body = "AKIAIKSA47YZNJAY2H6A\nAKIAJM4DOPAAJWLUJ2PQ"
        self.assertTrue(self.analyzer.match(self.paste))
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("AKIAIKSA47YZNJAY2H6A", match[0])
        self.assertEqual("AKIAJM4DOPAAJWLUJ2PQ", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid character '-'
        self.paste.body = "AKIAIX2GUZJMJFD-ON4A"
        self.assertFalse(self.analyzer.match(self.paste))

        # Different prefix
        self.paste.body = "AKIBIX2GUZJMJFDZON4A"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "AKIAIX2GUZJMJFDZON4"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "AKIAIX2GUZJMJFDZON4AA"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

from .basicaction import BasicAction
from .databaseaction import DatabaseAction
from .discordaction import DiscordAction
from .emailaction import EmailAction
from .ircaction import IrcAction
from .genericaction import GenericAction
from .logaction import LogAction
from .mispaction import MISPAction
from .savefileaction import SaveFileAction
from .savejsonaction import SaveJSONAction
from .telegramaction import TelegramAction
from .twitteraction import TwitterAction

__all__ = [
    "BasicAction",
    "SaveFileAction",
    "TelegramAction",
    "LogAction",
    "GenericAction",
    "DatabaseAction",
    "SaveJSONAction",
    "TwitterAction",
    "DiscordAction",
    "IrcAction",
    "MISPAction",
    "EmailAction"
    ]



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.shahashanalyzer import SHAHashAnalyzer


class TestSHAHashAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = SHAHashAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # obtained by: $ echo "foo" | shasum
        self.paste.body = "f1d2d2f924e986ac86fdf7b36c94bcdf32beec15"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by: $ echo "foo" | shasum -a 224
        self.paste.body = "e7d5e36e8d470c3e5103fedd2e4f2aa5c30ab27f6629bdc3286f9dd2"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by: $ echo "foo" | shasum -a 256
        self.paste.body = "b5bb9d8014a0f9b1d61e21e796d78dccdf1352f23cd32812f4850b878ae4944c"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by: $ echo "foo" | shasum -a 384
        self.paste.body = "8effdabfe14416214a250f935505250bd991f106065d899db6e19bdc8bf648f3ac0f1935c4f65fe8f798289b1a0d1e06"
        self.assertTrue(self.analyzer.match(self.paste))

        # obtained by: $ echo "foo" | shasum -a 512
        self.paste.body = "0cf9180a764aba863a67b6d72f0918bc131c6772642cb2dce5a34f0a702f9470ddc2bf125c12198b1995c233c34b4afd346c54a2334c350a948a51b6e8b4e6b6"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "my super cool hash is f1d2d2f924e986ac86fdf7b36c94bcdf32beec15 and here's some more text"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline-separated valid hashes
        self.paste.body = "f1d2d2f924e986ac86fdf7b36c94bcdf32beec15\ne7d5e36e8d470c3e5103fedd2e4f2aa5c30ab27f6629bdc3286f9dd2"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid character 'g'
        self.paste.body = "1234567890abcdefg"
        self.assertFalse(self.analyzer.match(self.paste))

        # MD5 Hash
        self.paste.body = "9e107d9d372bb6826bd81d3542a419d6"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid SHA hash length
        self.paste.body = 41 * "a"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = 513 * "a"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

from .dictwrapper import DictWrapper
from .listify import listify
from .request import Request
from .templatingengine import TemplatingEngine
from .threadingutils import start_thread, join_threads
from .network import enforce_ip_version

__all__ = ["Request", "DictWrapper", "start_thread", "join_threads", "TemplatingEngine", "listify", "enforce_ip_version"]



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.ipv4addressanalyzer import IPv4AddressAnalyzer


class TestIPv4AddressAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = IPv4AddressAnalyzer([])
        self.paste = mock.Mock()

    def test_positive(self):
        self.paste.body = "255.255.255.255"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "255.255.255.0"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "10.0.0.1"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "127.0.0.1"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "This is my private IP: 127.0.0.1 - this is secret!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("127.0.0.1", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "This is my private IP: 127.0.0.1 - this is secret! Also my server's IP is 178.55.12.16"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("127.0.0.1", match[0])
        self.assertEqual("178.55.12.16", match[1])

    def test_negative(self):
        self.paste.body = "111.123.1232.123"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "1111.123.123.123"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "111.123.999.123"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "111.123.1232.123"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .adobekeyanalyzer import AdobeKeyAnalyzer
from .alwaystrueanalyzer import AlwaysTrueAnalyzer
from .awsaccesskeyanalyzer import AWSAccessKeyAnalyzer
from .awssecretkeyanalyzer import AWSSecretKeyAnalyzer
from .awssessiontokenanalyzer import AWSSessionTokenAnalyzer
from .azuresubscriptionkeyanalyzer import AzureSubscriptionKeyAnalyzer
from .base64analyzer import Base64Analyzer
from .base64asciianalyzer import Base64AsciiAnalyzer
from .basicanalyzer import BasicAnalyzer
from .battlenetkeyanalyzer import BattleNetKeyAnalyzer
from .bcrypthashanalyzer import BcryptHashAnalyzer
from .creditcardanalyzer import CreditCardAnalyzer
from .databasedumpanalyzer import DatabaseDumpAnalyzer
from .dbconnstringanalyzer import DBConnAnalyzer
from .emailpasswordpairanalyzer import EmailPasswordPairAnalyzer
from .facebookaccesstokenanalyzer import FacebookAccessTokenAnalyzer
from .genericanalyzer import GenericAnalyzer
from .googleapikeyanalyzer import GoogleApiKeyAnalyzer
from .googleoauthkeyanalyzer import GoogleOAuthKeyAnalyzer
from .hashanalyzer import HashAnalyzer
from .ibananalyzer import IBANAnalyzer
from .logicalanalyzers import AndAnalyzer
from .logicalanalyzers import LogicalBaseAnalyzer
from .logicalanalyzers import OrAnalyzer
from .mailanalyzer import MailAnalyzer
from .mailchimpapikeyanalyzer import MailChimpApiKeyAnalyzer
from .md5hashanalyzer import MD5HashAnalyzer
from .megalinkanalyzer import MegaLinkAnalyzer
from .microsoftkeyanalyzer import MicrosoftKeyAnalyzer
from .originkeyanalyzer import OriginKeyAnalyzer
from .pastebinurlanalyzer import PastebinURLAnalyzer
from .phonenumberanalyzer import PhoneNumberAnalyzer
from .privatekeyanalyzer import PrivateKeyAnalyzer
from .regexanalyzer import RegexAnalyzer
from .shahashanalyzer import SHAHashAnalyzer
from .slacktokenanalyzer import SlackTokenAnalyzer
from .slackwebhookanalyzer import SlackWebhookAnalyzer
from .steamkeyanalyzer import SteamKeyAnalyzer
from .stripeapikeyanalyzer import StripeApiKeyAnalyzer
from .uplaykeyanalyzer import UplayKeyAnalyzer
from .urlanalyzer import URLAnalyzer
from .wordanalyzer import WordAnalyzer

__all__ = [
    "AdobeKeyAnalyzer",
    "AlwaysTrueAnalyzer",
    "AndAnalyzer",
    "AWSSecretKeyAnalyzer",
    "AWSAccessKeyAnalyzer",
    "AWSSessionTokenAnalyzer",
    "AzureSubscriptionKeyAnalyzer",
    "Base64Analyzer",
    "Base64AsciiAnalyzer",
    "BasicAnalyzer",
    "BattleNetKeyAnalyzer",
    "BcryptHashAnalyzer",
    "CreditCardAnalyzer",
    "DatabaseDumpAnalyzer",
    "DBConnAnalyzer",
    "EmailPasswordPairAnalyzer",
    "FacebookAccessTokenAnalyzer",
    "GenericAnalyzer",
    "GoogleApiKeyAnalyzer",
    "GoogleOAuthKeyAnalyzer",
    "HashAnalyzer",
    "IBANAnalyzer",
    "LogicalBaseAnalyzer",
    "MailAnalyzer",
    "MailChimpApiKeyAnalyzer",
    "MD5HashAnalyzer",
    "MegaLinkAnalyzer",
    "MicrosoftKeyAnalyzer",
    "OrAnalyzer",
    "OriginKeyAnalyzer",
    "PastebinURLAnalyzer",
    "PhoneNumberAnalyzer",
    "PrivateKeyAnalyzer",
    "RegexAnalyzer",
    "SHAHashAnalyzer",
    "SlackTokenAnalyzer",
    "SlackWebhookAnalyzer",
    "SteamKeyAnalyzer",
    "StripeApiKeyAnalyzer",
    "WordAnalyzer",
    "UplayKeyAnalyzer",
    "URLAnalyzer",
    ]



# -*- coding: utf-8 -*-

import unittest

from pastepwn import Paste
from pastepwn.util.templatingengine import TemplatingEngine


class TestTemplatingEngine(unittest.TestCase):

    def setUp(self):
        """Sets up the test case"""
        p = {"scrape_url": "https://scrape.pastebin.com/api_scrape_item.php?i=0CeaNm8Y",
             "full_url": "https://pastebin.com/0CeaNm8Y",
             "date": "1442911802",
             "key": "0CeaNm8Y",
             "size": "890",
             "expire": "1442998159",
             "title": "Once we all know when we goto function",
             "syntax": "java",
             "user": "admin",
             "body": "This is a test for pastepwn"}

        self.p = p
        self.paste = Paste(p.get("key"),
                           p.get("title"),
                           p.get("user"),
                           p.get("size"),
                           p.get("date"),
                           p.get("expire"),
                           p.get("syntax"),
                           p.get("scrape_url"),
                           p.get("full_url"))

    def test_fill_template(self):
        """Checks if templating engine inserts paste data correctly into the template"""
        analyzer_name = "TestAnalyzer"
        template = "Matched paste '${key}' by analyzer '${analyzer_name}'. URL is: '${full_url}'"
        expected = "Matched paste '{0}' by analyzer '{1}'. URL is: '{2}'".format(self.p.get("key"), analyzer_name, self.p.get("full_url"))
        result = TemplatingEngine.fill_template(paste=self.paste, analyzer_name=analyzer_name, template_string=template)
        self.assertEqual(expected, result, msg="Filled template string is not the same as the expected result!")

    def test_fill_template_matches(self):
        """Checks if templating engine inserts the matches correctly into the template"""
        template = "Matches are: ${matches}"
        expected = "Matches are: +123456789\n+987654321"
        matches = ["+123456789", "+987654321"]
        result = TemplatingEngine.fill_template(paste=self.paste, analyzer_name=None, template_string=template, matches=matches)
        self.assertEqual(expected, result, msg="Filled template string is not the same as the expected result!")

    def test_fill_template_kwarg(self):
        """Checks if templating engine inserts arbitrary data via kwargs into the template"""
        template = "Completely new parameter ${random_param} unrelated to paste data can be ${ins} into this string"
        expected = "Completely new parameter 'pastepwnIsCool' unrelated to paste data can be inserted into this string"

        result = TemplatingEngine.fill_template(paste=self.paste, analyzer_name=None, template_string=template, random_param="'pastepwnIsCool'",
                                                ins="inserted"
                                                )
        self.assertEqual(expected, result, msg="Filled template string is not the same as the expected result!")

    def test_fill_template_missing_param(self):
        """Checks if templating engine correctly handles nonexistent params"""
        template = "The nonexistent parameter ${i_do_not_exist} stays the same!"
        expected = "The nonexistent parameter ${i_do_not_exist} stays the same!"

        result = TemplatingEngine.fill_template(paste=self.paste, analyzer_name=None, template_string=template)
        self.assertEqual(expected, result, msg="Filled template string is not the same as the expected result!")

    if __name__ == "__main__":
        unittest.main()



# -*- coding: utf-8 -*-

from .ipnotregisterederror import IPNotRegisteredError
from .pastedeletedexception import PasteDeletedException
from .pasteemptyexception import PasteEmptyException
from .pastenotreadyexception import PasteNotReadyException

__all__ = ["IPNotRegisteredError", "PasteNotReadyException", "PasteDeletedException", "PasteEmptyException"]



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class DBConnAnalyzer(RegexAnalyzer):
    """Analyzer to match on email addresses via regex"""
    name = "DBConnAnalyzer"

    def __init__(self, actions):
        # https:// should be ignored
        regex = r"(\b(?!http(?:s)?\b)\w+[a-zA-Z]+:\/\/[a-zA-z0-9.:,\-@]+)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class DatabaseDumpAnalyzer(RegexAnalyzer):
    """Analyzer to match database dump"""
    name = "DatabaseDumpAnalyzer"

    def __init__(self, actions):
        """
        Analyzer to match database dump
        :param actions: A single action or a list of actions to be executed on every paste
        """
        # This regex match the columns of a database
        regex = r"\((?:(?:`\w+`|\d)(?:\s?)+,(?:\s?)+)+(?:`\w+`|\d)\)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.googleoauthkeyanalyzer import GoogleOAuthKeyAnalyzer


class TestGoogleOAuthKeyAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = GoogleOAuthKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # google key dump
        self.paste.body = "6243-jhgcawesuycgaweiufyugfaiwyesfbaw.apps.googleusercontent.com"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "6243-IUFHERIUFHASOEIRUFGHDOZIFUGVDHSF.apps.googleusercontent.com"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "6243-18723612873612873621367128736128.apps.googleusercontent.com"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "1-jhgcawesuycgaweiufyugfaiwyesfbaw.apps.googleusercontent.com"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "6242345234234234234234234233-jhgcawesuycgaweiufyugfaiwyesfbaw.apps.googleusercontent.com"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "6243-jhgcawesuycgaweiufyugfaisfbaw.apps.googleusercontent.com"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid numbers
        self.paste.body = "-jhgcawesuycgaweiufyugfaiwyesfbaw.apps.googleusercontent.com"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid domain
        self.paste.body = "6243-jhgcawesuycgaweiufyugfaiwyesfbaw.apps.google.com"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SteamKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match steam keys via regex."""
    name = "SteamKeyAnalyzer"

    def __init__(self, actions):
        """
        The analyzer matches 5-5-5-5-5 and 5-5-5 pattern steam keys
        Lower case keys seem to be forbidden
        :param actions: A single action or a list of actions to be executed on every paste
        """
        regex = r"\b(?<!-)[A-Z0-9]{5}-[A-Z0-9]{5}-[A-Z0-9]{5}(?:-[A-Z0-9]{5}-[A-Z0-9]{5})?\b(?!-)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.pastebinurlanalyzer import PastebinURLAnalyzer


class TestPastebinURLAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = PastebinURLAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        self.paste.body = "https://pastebin.com/xyz"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "https://pastebin.com/xyz/"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "http://pastebin.com/xyz"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "http://pastebin.com/xyz/"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "https://pastebin.com/xyz  "
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "www.pastebin.com/xyz"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "pastebin.com/xyx"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "This is a pastebin URL: pastebin.com/ya249asd - and this is a test"

    def test_match_negative(self):
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "https://google.com/xyz"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "xyzpastebin.com/k"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "https://pastebin.com/"
        self.assertFalse(self.analyzer.match(self.paste))

    if __name__ == "__main__":
        unittest.main()



# -*- coding: utf-8 -*-
from string import Template

from pastepwn.util import DictWrapper


class TemplatingEngine(object):
    """Wrapper class around the python templating feature"""

    @staticmethod
    def fill_template(paste, analyzer_name, template_string, matches=None, **kwargs):
        """
        Returns a templated text with paste contents inserted into the template string
        Use ${key_name} in the template_string to insert paste contents into it
        :param paste: A paste which serves as the source for template filling
        :param analyzer_name: Name of the analyzer
        :param template_string: A template string describing how the variables should be filled in
        :param matches: A list of matches that was returned from the analyzer
        :return: Filled template
        """
        paste_dict = paste.to_dict()
        paste_dict["analyzer_name"] = analyzer_name

        if matches is None:
            paste_dict["matches"] = ""
        else:
            # When there are elements in the matches object, we want them to be formatted as single string
            matches_str = "\n".join(matches)
            paste_dict["matches"] = matches_str

        # Possibility to insert own/custom values into the paste_dict thus gives more control over the template string
        for name, value in kwargs.items():
            paste_dict[name] = value

        # Fallback if the template string is empty or non existent
        if template_string is None or template_string == "":
            template_string = "New paste matched by analyzer '${analyzer_name}' - Link: ${full_url}\n\nMatches:\n${matches}"

        template = Template(template_string)
        return template.safe_substitute(DictWrapper(paste_dict))



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class PrivateKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match private keys beginnigs via regex"""
    name = "PrivateKeyAnalyzer"

    def __init__(self, actions):
        regex = r"[\-]{4,5}(?: )?BEGIN(?: [A-Z0-9]+)? PRIVATE KEY(?: BLOCK)?(?: )?[\-]{4,5}[\s\w\:\"\-\/\+\=]+?[\-]{4,5}(?: )?END(?: [A-Z0-9]+)? " \
                r"PRIVATE KEY( BLOCK)?(?: )?[\-]{4,5}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
# Do not mess with the order of the imports
# Otherwise there will be circular imports -> bad

from .paste import Paste
from .actionhandler import ActionHandler
from .scrapinghandler import ScrapingHandler
from .pastedispatcher import PasteDispatcher
from .pastepwn import PastePwn

__all__ = ["PastePwn", "ActionHandler", "ScrapingHandler", "PasteDispatcher", "Paste"]



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class MicrosoftKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match Microsoft Licensing Keys"""

    def __init__(self, action):
        """
        Analyzer to match Microsoft Licensing Keys
        :param action: Single action or list of actions to be executed when a paste matches
        """
        # Tested Regex against https://pastebin.com/r3QdpFJf
        regex = r"\b(?<!-)[2346789BCDFGHJKMNPQRTVWXY]{5}(?:-[2346789BCDFGHJKMNPQRTVWXY]{5}){4}\b(?!-)"

        super().__init__(action, regex)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class StripeApiKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match Stripe API keys"""

    def __init__(self, actions):
        regex = r"(?:r|s)k_(?:live|test)_[0-9a-zA-Z]{24}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-

import unittest
from unittest.mock import Mock, patch

from pastepwn.actions import SaveJSONAction


class TestSaveJSONAction(unittest.TestCase):

    def setUp(self):
        """Sets up the test case"""
        self.action = SaveJSONAction(path="")
        self.paste = Mock()
        self.path_mock = Mock()
        self.path_mock.__truediv__ = Mock(side_effect=self.side_effect)
        self.path_mock.exists = Mock(return_value=True)

    @staticmethod
    def side_effect(path):
        return path

    def test_get_file_content(self):
        """Check if the content of the file is returned correctly"""
        self.paste.to_dict = Mock(return_value={"test": "content", "another": "item"})
        content_string = """{"test": "content", "another": "item"}"""
        content = self.action.get_file_content(self.paste, "", [])
        self.assertEqual(content_string, content)

    @patch("builtins.open")
    @patch("pastepwn.actions.savejsonaction.json")
    def test_file_ending(self, json_mock, open_mock):
        """Check that the file ending is actually json, not txt"""
        json_mock.dumps = Mock(return_value="json content!")

        self.action.path = self.path_mock

        self.paste.key = Mock()
        self.paste.key.__repr__ = Mock(return_value="123456")
        self.action.perform(self.paste)

        open_mock.assert_called_with("123456.json", "w", encoding="utf-8")
        open_mock().__enter__().write.assert_called_with("json content!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.azuresubscriptionkeyanalyzer import AzureSubscriptionKeyAnalyzer


class TestAzureSubscriptionKeyAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = AzureSubscriptionKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # google key dump
        self.paste.body = "74796abfc83c49bba7458746266abd28"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "00000000000000000000000000000000"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "a32324343543bcdea32324343543bcde"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        # key appearing in the middle of a string
        self.paste.body = "my azure key is: a32324343543bcdea32324343543bcde, aaaa"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("a32324343543bcdea32324343543bcde", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "There are multiple keys: a32324343543bcdea32324343543bcde, " \
                          "aaaand also aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("a32324343543bcdea32324343543bcde", match[0])
        self.assertEqual("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid characters
        self.paste.body = "aaaaaaaaaaaaaaa-aaaaaaaaaaaaaaaa"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid casing
        self.paste.body = "AAAAAAAABBBB11111111111111111111"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "AAAAAAAAAA"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.logicalanalyzers import LogicalBaseAnalyzer


class TestLogicalBaseAnalyzer(unittest.TestCase):
    def setUp(self):
        self.paste = mock.Mock()

    def test_exception(self):
        analyzer = LogicalBaseAnalyzer([], [])
        self.assertRaises(NotImplementedError, analyzer.match, mock.Mock())

    def test_actions_present(self):
        action = mock.MagicMock(spec=BasicAction)
        analyzer = LogicalBaseAnalyzer(action, None)
        self.assertEqual([action], analyzer.actions)

    def test_analyzers_present(self):
        analyzer = LogicalBaseAnalyzer(None, self.paste)
        self.assertEqual([self.paste], analyzer.analyzers)

    def test_merge_actions(self):
        action1 = mock.Mock()
        action2 = mock.Mock()
        action3 = mock.Mock()

        analyzer1 = mock.Mock()
        analyzer1.actions = [action1, action2]
        analyzer2 = mock.Mock()
        analyzer2.actions = [action3]

        analyzer = LogicalBaseAnalyzer(analyzers=[analyzer1, analyzer2], actions=[], merge_actions=True)
        self.assertEqual(3, len(analyzer.actions), "Wrong amount of actions in LogicalBaseAnalyzer!")
        self.assertEqual([action1, action2, action3], analyzer.actions, "Actions do not match!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.microsoftkeyanalyzer import MicrosoftKeyAnalyzer


class TestMicrosoftKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = MicrosoftKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # microsoft key dump
        self.paste.body = "2YM6W-NXCWB-RX6F2-QFVDW-J8C9V"
        self.assertTrue(self.analyzer.match(self.paste))

        # microsoft key dump
        self.paste.body = "6N667-BMRDR-T2WMM-2RMQ9-DYF3H"
        self.assertTrue(self.analyzer.match(self.paste))

        # microsoft key dump
        self.paste.body = "88PNQ-KF99B-VJG64-R28RW-D9JQH"
        self.assertTrue(self.analyzer.match(self.paste))

        # microsoft key dump
        self.paste.body = "BNQWW-X4422-FCXD8-JPT37-PWC9V"
        self.assertTrue(self.analyzer.match(self.paste))

        # part of a sentence
        self.paste.body = "Hey, I have your key right here: GC288-NVPMT-GWF2M-76228-W42DH!"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline seperated microsoft key
        self.paste.body = "N3B2Q-F9GQF-PTDBG-KKRR7-7QTXV\nN6C37-P8DW3-YVG9X-BRW77-9BQ67"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertTrue("N3B2Q-F9GQF-PTDBG-KKRR7-7QTXV", match[0])
        self.assertTrue("N6C37-P8DW3-YVG9X-BRW77-9BQ67", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "PNT6B-DKH2Q-GW4J2-DDT6T-PDHT76"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "PNT6B-DKH2Q-GW4J2-DDT6T-PDHT"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid Characters
        self.paste.body = "A78QN-Y8XXR-MKFK7-Q4R6C-Q3TXV"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-


class Paste(object):
    """Representation of a paste object used for example by pastebin"""

    def __init__(self, key, title, user, size, date, expire, syntax, scrape_url, full_url):
        self.key = key
        self.title = title
        self.user = user
        self.size = size
        self.date = date
        self.expire = expire
        self.syntax = syntax
        self.scrape_url = scrape_url
        self.full_url = full_url
        self.body = ""

    def set_body(self, body):
        """
        Sets the body of the paste
        :param body: String containing the content of the paste
        :return:
        """
        self.body = str(body or "")

    def __str__(self):
        return str(self.to_dict())

    __repr__ = __str__

    def to_dict(self):
        """
        Generates a dict out of the paste
        :return: Paste as dict
        """
        return self.__dict__.copy()



# -*- coding: utf-8 -*-
import logging

from .basicanalyzer import BasicAnalyzer


class GenericAnalyzer(BasicAnalyzer):
    """Analyzer to pass a function pointer to, in order to create an analyzer on the fly"""
    name = "GenericAnalyzer"

    def __init__(self, actions, match_func, verify_func=None):
        super().__init__(actions)

        if match_func is None:
            raise ValueError("Function to be called cannot be None")
        elif not callable(match_func):
            raise ValueError("Function you provided isn't callable")

        if verify_func is not None and not callable(verify_func):
            raise ValueError("Verify function you provided isn't callable")

        self.verify_func = verify_func
        self.match_func = match_func

    def verify(self, results):
        """Method to perform additional checks to test if the matches are actually valid"""
        # If no custom verify method is specified, we return True
        if self.verify_func is None:
            return results

        # Otherwise we try to execute the verify method
        try:
            return self.verify_func(results)
        except Exception as e:
            logging.getLogger(__name__).warning("Executing custom verify function '{}' raised an exception! {}".format(self.verify_func.__name__, e))

    def match(self, paste):
        """Run the passed function and return its return value"""
        try:
            results = self.match_func(paste)
        except Exception as e:
            logging.getLogger(__name__).warning("Executing custom match function '{}' raised an exception! {}".format(self.match_func.__name__, e))
            return False

        if not self.verify(results):
            return False

        return results



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.epickeyanalyzer import EpicKeyAnalyzer


class TestEpicKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = EpicKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # Epic key dump
        self.paste.body = "1GZJQ-DW7QX-SB4DC-THDW2"
        self.assertTrue(self.analyzer.match(self.paste))

        # Epic key dump
        self.paste.body = "KWETC-MK13P-4DO0N-VHT62"
        self.assertTrue(self.analyzer.match(self.paste))

        # Epic key dump
        self.paste.body = "MB9KG-TGXBJ-X8OXE-J7PIF"
        self.assertTrue(self.analyzer.match(self.paste))

        # Epic key dump
        self.paste.body = "OMYCF-Q9VYL-4FQEG-8F3XV"
        self.assertTrue(self.analyzer.match(self.paste))

        # part of a sentence
        self.paste.body = "Look it's FORTNITE! UGCTH-FH42S-OH98G-QHFZA"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline seperated Epic keys
        self.paste.body = "87C6Y-XIV2I-C3RJZ-B1SVZ\nQ8AQT-APT3F-MO7QU-KPE96"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length in section
        self.paste.body = "Q8A5QT-APFT3F-MO74QU-KPEL96"
        self.assertFalse(self.analyzer.match(self.paste))

        # Too many segments
        self.paste.body = "1GZJQ-DW7QX-SB4DC-THDW2-1A2B3"
        self.assertFalse(self.analyzer.match(self.paste))

        # No separator
        self.paste.body = "OAPMCSEU6N7FFZ72AM5E"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import asyncio
import json
import logging
import sys

from pastepwn.util import Request
from pastepwn.util import TemplatingEngine
from .basicaction import BasicAction

websockets_available = True
try:
    import websockets
except ImportError:
    websockets = None
    websockets_available = False


class DiscordAction(BasicAction):
    """Action to send a Discord message to a certain webhook or channel."""
    name = "DiscordAction"

    def __init__(self, webhook_url=None, token=None, channel_id=None, template=None):
        """Action to send a Discord message to a certain webhook or channel.
        Either the webhook parameter or the token & channel_id parameters are needed.

        1) You can setup a webhook in the discord server settings. A webhook is tied to one server & text channel
        > https://support.discordapp.com/hc/en-us/articles/228383668-Intro-to-Webhooks


        2) Setup a discord bot (token) in the developer portal:
        > https://discordapp.com/developers/applications/
        After creating an app you can obtain the token by going to
        > https://discordapp.com/developers/applications/{your_app_id}/bot
        Format:
        > NTI5MzI1MzY4OTAyMDI1MjI3.DwvNFQ.5aNKUvYlAKqKKq6UJ1fRiARKNXQ

        3) Obtain the channel_id (18 digit number):
        > User Settings > Appearance > Enable Developer Mode and after that right click on any text channel to copy the ID

        :param webhook_url: The url obtained from the server settings
        :param token: A bot token obtained from the developer portal
        :param channel_id: The channel ID of a text channel you want to send messages into
        :param template: A template string describing how the paste variables should be filled in
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.bot_available = True

        if websockets is None or not websockets_available or (sys.version_info.major == 3 and sys.version_info.minor >= 10):
            self.logger.warning("Could not import 'websockets' module. So you can only use webhooks for discord.")
            self.bot_available = False

        self.webhook_url = webhook_url
        if webhook_url is None:
            # When there is no webhook_url, we need both token and channel_id
            if token is None or channel_id is None:
                raise ValueError("Invalid arguments: requires either webhook_url or token+channel_id arguments")

            if not self.bot_available:
                raise NotImplementedError("You can't use bot functionality without the 'websockets' module. Please import it or use webhooks!")

            self.token = token
            self.channel_id = channel_id
            self.identified = False

        self.template = template

    @asyncio.coroutine
    def _identify(self, ws_url):
        """Connect to the Discord Gateway to identify the bot."""
        # Docs: https://discordapp.com/developers/docs/topics/gateway#connecting-to-the-gateway
        # Open connection to the Discord Gateway
        if websockets is None:
            raise ImportError("Couldn't import websockets!")

        socket = yield from websockets.connect("{0}/?v=6&encoding=json".format(ws_url))
        try:
            # Receive Hello
            hello_str = yield from socket.recv()
            hello = json.loads(hello_str)
            if hello.get("op") != 10:
                self.logger.warning("[ws] Expected Hello payload but received %s", hello_str)

            # Send heartbeat and receive ACK
            yield from socket.send(json.dumps({"op": 1, "d": {}}))
            ack_str = yield from socket.recv()
            ack = json.loads(ack_str)

            # https://discord.com/developers/docs/topics/opcodes-and-status-codes#gateway-gateway-opcodes
            heartbeat_ack = 11
            if ack.get("op") != heartbeat_ack:
                self.logger.warning("[ws] Expected Heartbeat ACK payload but received %s", ack_str)

            # Identify
            payload = {
                "token": self.token,
                "properties": {
                    "$os": sys.platform,
                    "$browser": "pastepwn",
                    "$device": "pastepwn"
                    }
                }
            yield from socket.send(json.dumps({"op": 2, "d": payload}))

            # Receive READY event
            ready_str = yield from socket.recv()
            ready = json.loads(ready_str)
            if ready.get("t") != "READY":
                self.logger.warning("[ws] Expected READY event but received %s", ready_str)
        finally:
            # Close websocket connection
            yield from socket.close()

    def initialize_gateway(self):
        """Initialize the bot token so Discord identifies it properly."""
        if self.webhook_url is not None:
            raise NotImplementedError("Gateway initialization is only necessary for bot accounts.")

        # Call Get Gateway Bot to get the websocket URL
        # https://discordapp.com/developers/docs/reference#authentication
        r = Request()
        r.headers = {"Authorization": "Bot {}".format(self.token)}
        res = json.loads(r.get("https://discordapp.com/api/gateway/bot"))
        ws_url = res.get("url")

        # Start websocket client
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(self._identify(ws_url))
        self.identified = True

    def perform(self, paste, analyzer_name=None, matches=None):
        """Send a message via Discord to a specified channel, without checking for errors"""
        r = Request()
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)

        if self.webhook_url is not None:
            # Send to a webhook (no authentication)
            url = self.webhook_url
        else:
            # Send through Discord bot API (header-based authentication)
            url = "https://discordapp.com/api/channels/{0}/messages".format(self.channel_id)
            r.headers = {"Authorization": "Bot {}".format(self.token)}

        res = r.post(url, {"content": text})
        if res == "":
            # If the response is empty, skip further execution
            return

        res = json.loads(res)

        # https://discord.com/developers/docs/topics/opcodes-and-status-codes#json-json-error-codes
        unauthorized_code = 40001
        if res.get("code") == unauthorized_code and self.bot_available and self.webhook_url is None and not self.identified:
            # Unauthorized access, bot token hasn't been identified to Discord Gateway
            self.logger.info("Accessing Discord Gateway to initialize token")
            self.initialize_gateway()
            # Retry action
            self.perform(paste, analyzer_name=analyzer_name)



# -*- coding: utf-8 -*-
from .base64analyzer import Base64Analyzer
from base64 import b64decode
import binascii


class Base64AsciiAnalyzer(Base64Analyzer):
    """Analyzer to match base64 strings which decode to valid ASCII"""
    name = "Base64AsciiAnalyzer"

    def __init__(self, actions, min_len=1, decode=False):
        super().__init__(actions, min_len)
        self.decode = decode

    def verify(self, results):
        """Method to determine if found base64 decodes to valid ASCII"""
        # find valid base64 strings with the parent class
        validated_strings = super().verify(results)

        # go through each base64 string and attempt to decode
        base64_ascii_strings = []

        for validated_string in validated_strings:
            # Check if the string is valid base64
            try:
                decoded_string = b64decode(validated_string)
            except binascii.Error:
                # The string is no valid base64
                continue

            # Check if the valid base64 decodes to plain ascii
            try:
                b64_ascii_string = decoded_string.decode("ascii")
            except UnicodeDecodeError:
                continue

            if self.decode:
                base64_ascii_strings.append(b64_ascii_string)
            else:
                base64_ascii_strings.append(validated_string)

        return base64_ascii_strings



# -*- coding: utf-8 -*-
from .urlanalyzer import URLAnalyzer


class PastebinURLAnalyzer(URLAnalyzer):
    """Analyzer to match Pastebin URLs via regex"""
    name = "PastebinURLAnalyzer"

    def __init__(self, actions, resolve=False):
        regex = r"((?:https?:\/\/)?pastebin\.com\/[a-zA-Z0-9]{3,})"
        super().__init__(actions, regex, resolve)



# -*- coding: utf-8 -*-


# https://stackoverflow.com/questions/19799609/leaving-values-blank-if-not-passed-in-str-format
class DictWrapper(dict):
    """A wrapper around dicts which returns the key as string when missing.
    Used for the templating engine
    """

    def __missing__(self, key):
        return "${{{0}}}".format(key)



# -*- coding: utf-8 -*-

import os
import random
import string
import unittest

from pastepwn import Paste
from pastepwn.database import MongoDB


@unittest.skipIf(os.environ.get("CI"), "Skipping this test on CI.")
class MongoDBTest(unittest.TestCase):

    def setUp(self):
        rand_text = []
        for _ in range(3):
            rand_text.append("".join(random.choice(string.ascii_uppercase + string.ascii_lowercase + string.digits) for _ in range(8)))

        p = {"scrape_url": "https://scrape.pastebin.com/api_scrape_item.php?i=" + rand_text[0],
             "full_url": "https://pastebin.com/" + rand_text[0],
             "date": "1442911802",
             "key": rand_text[0],
             "size": "890",
             "expire": "1442998159",
             "title": "Once we all know when we goto function",
             "syntax": "java",
             "user": "admin",
             "body": rand_text[1:],
             }

        self.p = p
        self.paste = Paste(p.get("key"),
                           p.get("title"),
                           p.get("user"),
                           p.get("size"),
                           p.get("date"),
                           p.get("expire"),
                           p.get("syntax"),
                           p.get("scrape_url"),
                           p.get("full_url")
                           )

        self.database = MongoDB(collectionname="pastepwn_test")

    def tearDown(self):
        self.database.db.drop_collection("pastepwn_test")

    def test_insert_same_key(self):
        # Insert a paste two times with the same body
        for body_text in self.p.get("body"):
            self.paste.set_body(body_text)
            self.database.store(self.paste)

        stored_paste = self.database.get(self.p.get("key"))
        comparison = self.p.get("body")[1]
        self.assertEqual(stored_paste.next().get("body"), comparison)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.creditcardanalyzer import CreditCardAnalyzer


class TestCreditCardAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = CreditCardAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized
        * CARDS IN ORDER *
        VISA
        MASTERCARD
        AMERICAN EXPRESS
        DINERS CLUB
        DISCOVER
        JCB
        """
        self.paste.body = "4556316812657526"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "4556 3168 1265 7526"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "5168441223630339"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "5168 4412 2363 0339"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "371642190784801"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "3716 421907 84801"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "30043277253249"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "3004 327725 3249"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "6011988461284820"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "6011 9884 6128 4820"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "3538684728624673"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "3538 6847 2862 4673"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "https://www.github.com"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "1234 5678 9101 12"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "123456789101112"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "51684+4122363339"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "+5168441223630"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "5368441223630339183239284"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.md5hashanalyzer import MD5HashAnalyzer


class TestMD5HashAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = MD5HashAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # md5 hash of 128 character string: ndq=S)t<5dgQhD7rCyAZ<?k,b&Zh-t^@'d=kaf:tK$PH\8HSzt%q9pX&RsGuYAcnmy9Rh!'(9sWt.d&\hS,LSA4>#Gn@sYn``5Tt6X`h'Hn_Y-"F>DkGr'\/:4kLF:D"
        self.paste.body = "5812c57a01b5efa30af09d4f9388e50d"
        self.assertTrue(self.analyzer.match(self.paste))

        # md5 hash of 64 character string: sa.tpMvv#J{q)tZfz,W[5Hq*Yz%kN(,8j)p>'g["d^mSLHkD)gZLVxk/,}#aVxv*
        self.paste.body = "372f202bdf0f16dfa41c101fc6a41695"
        self.assertTrue(self.analyzer.match(self.paste))

        # md5 hash of 32 character string: zw8hspDAZ(\w]K/v~yZaa_m$6awgF4rj
        self.paste.body = "2287c6975240f865a9945ec9bcc0eead"
        self.assertTrue(self.analyzer.match(self.paste))

        # md5 hash of 16 character string: cY4>szK'>(?48tz=
        self.paste.body = "f2195cea00cc796676f77b9d19473f7a"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = "my super cool hash is 2287c6975240f865a9945ec9bcc0eead and here's some more text"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline-separated valid hashes
        self.paste.body = "2287c6975240f865a9945ec9bcc0eead\n372f202bdf0f16dfa41c101fc6a41695"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid character 'g'
        self.paste.body = "f2195cea00cc796676f77b9d19473f7g"
        self.assertFalse(self.analyzer.match(self.paste))

        # SHA Hash
        self.paste.body = "f1d2d2f924e986ac86fdf7b36c94bcdf32beec15"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid MD5 hash length
        self.paste.body = "f2195cea00cc796676f77b9d19473f7"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SlackTokenAnalyzer(RegexAnalyzer):
    """Analyzer to match the content of a paste via regular expressions that look like slack tokens."""

    name = "SlackTokenAnalyzer"

    def __init__(self, actions):
        """Analyzer which matches a slack token

        :param actions: A single action or a list of actions to be executed on every paste
        """
        regex = r"xox[pboa]-[0-9]{12}-[0-9]{12}-[0-9]{12}-[a-z0-9]{32}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import hashlib
import re
from .regexanalyzer import RegexAnalyzer


class HashAnalyzer(RegexAnalyzer):
    """Analyzer to match multiple hashes of user-given passwords."""
    name = "HashAnalyzer"

    def __init__(self, actions, passwords, algorithms=None):
        """Hashes given passwords with multiple algorithms and matches the output.

        :param actions: A single action or a list of actions to be executed on every paste
        :param passwords: A single password or a list of passwords to hash, as bytes
        :param algorithms: A list of algorithm names to use for hashing. This should be a subset
                           of hashlib.algorithms_available, and defaults to it.
        """
        # Make sure passwords is a list
        if isinstance(passwords, bytes):
            passwords = [passwords]

        # Build algorithm list
        if algorithms is None:
            algorithms = hashlib.algorithms_available
        else:
            algorithms = set(algorithms).intersection(hashlib.algorithms_available)

        if not algorithms:
            raise ValueError("No valid algorithm names specified")

        # Compute hashes with all algorithms
        hashes = []
        for hash_name in algorithms:
            hash_function = hashlib.new(hash_name)
            for password in passwords:
                hash_ = hash_function.copy()
                hash_.update(password)
                if hash_name == "shake_128":
                    digest = hash_.hexdigest(128)
                elif hash_name == "shake_256":
                    digest = hash_.hexdigest(256)
                else:
                    digest = hash_.hexdigest()
                hashes.append(digest)

        # Build regex
        regex = r"\b(%s)\b" % "|".join(hashes)
        super().__init__(actions, regex, re.IGNORECASE)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.dbconnstringanalyzer import DBConnAnalyzer


class TestDBConnAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = DBConnAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        self.paste.body = "cassandra://194.3.5.2"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("cassandra://194.3.5.2", self.analyzer.match(self.paste)[0])

        self.paste.body = "postgresql://user:secret@localhost"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("postgresql://user:secret@localhost", self.analyzer.match(self.paste)[0])

        self.paste.body = "mongodb://router1.example.com:27017"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("mongodb://router1.example.com:27017", self.analyzer.match(self.paste)[0])

        self.paste.body = "The db connection string is 'mongodb://router1.example.com:27017' but please keep this for yourself!"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("mongodb://router1.example.com:27017", self.analyzer.match(self.paste)[0])

    def test_match_multiple(self):
        self.paste.body = "The db connection string is 'mongodb://router1.example.com:27017' but please keep this for yourself! Also make sure to use " \
                          "postgresql://user:secret@localhost as postgres connection"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("mongodb://router1.example.com:27017", match[0])
        self.assertEqual("postgresql://user:secret@localhost", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "https://www.google.com"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "http://www.google.com"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import re

from .basicanalyzer import BasicAnalyzer


class PasteTitleAnalyzer(BasicAnalyzer):
    """Analyzer to match Paste titles via regex"""

    name = "PasteTitleAnalyzer"

    def __init__(self, actions, regex, flags=0, blacklist=None):
        super().__init__(actions, self.name)
        self.regex = re.compile(regex, flags)
        self.blacklist = blacklist or []

    def match(self, paste):
        """Match the title of a paste via regex. Return true if regex matches"""
        paste_title = paste.title or ""
        return self.regex.findall(paste_title)



# -*- coding: utf-8 -*-


def listify(obj):
    """Make sure the given object is a list

    :param obj: Any object - either None, a list of objects or a single object
    :return: The given object formatted as list
    """
    if obj is None:
        # When the object is None, an empty list will be returned
        return []
    elif isinstance(obj, list):
        # When the object is already a list, that list will be returned
        return obj

    # When a single object is passed to the method, a list with the
    # object as single item will be returned
    return [obj]



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class AdobeKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match Adobe License Keys"""

    def __init__(self, action):
        """Analyzer to match Adobe License Keys.
        :param action: Single action or list of actions to be executed when a paste matches
        """
        # Tested Regex against https://pastebin.com/fxWBGf8t
        regex = r"\b(?<!-)[0-9]{4}(?:-[0-9]{4}){5}\b(?!-)"

        super().__init__(action, regex)



# -*- coding: utf-8 -*-
import logging
import sys
from queue import Queue
from signal import signal, SIGINT, SIGTERM, SIGABRT
from threading import Event
from time import sleep

from pastepwn.actions import DatabaseAction
from pastepwn.analyzers import AlwaysTrueAnalyzer
from pastepwn.core import ScrapingHandler, ActionHandler, PasteDispatcher
from pastepwn.util import Request, enforce_ip_version
from pastepwn.scraping.pastebin import PastebinScraper


class PastePwn(object):
    """Represents an instance of the pastepwn core module"""

    def __init__(self, database=None, proxies=None, store_all_pastes=True, ip_version=None):
        """
        Basic PastePwn object handling the connection to pastebin and all the analyzers and actions
        :param database: Database object extending AbstractDB
        :param proxies: Dict of proxies as defined in the requests documentation
        :param store_all_pastes: Bool to decide if all pastes should be stored into the db
        :param ip_version: The IP version pastepwn should use (4|6)
        """
        self.logger = logging.getLogger(__name__)
        self.is_idle = False
        self.database = database
        self.paste_queue = Queue()
        self.action_queue = Queue()
        self.error_handlers = []
        self.onstart_handlers = []
        self.__exception_event = Event()
        self.__request = Request(proxies)  # initialize singleton

        # We are trying to enforce a certain version of the Internet Protocol
        enforce_ip_version(ip_version)

        # Usage of ipify to get the IP - Uses the X-Forwarded-For Header which might
        # lead to issues with proxies
        try:
            ip = self.__request.get("https://api.ipify.org")
        except Exception as e:
            ip = None
            self.logger.warning("Could not fetch public IP via ipify: {0}".format(e))

        if ip:
            self.logger.info("Your public IP is {0}".format(ip))

        self.scraping_handler = ScrapingHandler(paste_queue=self.paste_queue,
                                                exception_event=self.__exception_event)
        self.paste_dispatcher = PasteDispatcher(paste_queue=self.paste_queue,
                                                action_queue=self.action_queue,
                                                exception_event=self.__exception_event)
        self.action_handler = ActionHandler(action_queue=self.action_queue,
                                            exception_event=self.__exception_event)

        if self.database is not None and store_all_pastes:
            # Save every paste to the database with the AlwaysTrueAnalyzer
            self.logger.info("Database provided! Storing pastes in there.")
            database_action = DatabaseAction(self.database)
            always_true = AlwaysTrueAnalyzer(database_action)
            self.add_analyzer(always_true)
        elif store_all_pastes:
            self.logger.info("No database provided!")
        else:
            self.logger.info("Not storing all pastes!")

    def add_scraper(self, scraper, restart_scraping=False):
        """Adds a scraper to the list of scrapers. Scraping handler must be restarted for this to take effect.
        :param scraper: Instance of a BasicScraper
        :param restart_scraping: Indicates if the scraping handler should be restarted. Not setting this option results in your scraper not being started.
        :return: None
        """
        scraper.init_exception_event(self.__exception_event)
        self.scraping_handler.add_scraper(scraper, restart_scraping)

    def add_analyzer(self, analyzer):
        """Adds a new analyzer to the list of analyzers
        :param analyzer: Instance of a BasicAnalyzer
        :return: None
        """
        self.paste_dispatcher.add_analyzer(analyzer)

    def add_error_handler(self, error_handler):
        """
        Adds an error handler which will be called when an error happens
        :param error_handler: Callable to be called when an error happens
        :return: None
        """
        if not callable(error_handler):
            self.logger.error("The error handler you passed is not a function!")
            return

        self.error_handlers.append(error_handler)

    def add_onstart_handler(self, onstart_handler):
        """Add a function as onstart_handler"""
        if not callable(onstart_handler):
            self.logger.error("The onstart handler you passed is not a function!")
            return

        self.onstart_handlers.append(onstart_handler)

    def start(self):
        """Starts the pastepwn instance"""
        if self.__exception_event.is_set():
            self.logger.error("An exception occured. Aborting the start of PastePwn!")
            sys.exit(1)
        if not self.scraping_handler.scrapers:
            pastebinscraper = PastebinScraper()
            self.add_scraper(pastebinscraper, restart_scraping=True)
        self.scraping_handler.start()
        self.paste_dispatcher.start()
        self.action_handler.start()

        for onstart_handler in self.onstart_handlers:
            try:
                onstart_handler()
            except Exception as e:
                self.logger.error("Onstart handler %s failed with error: %s. Pastepwn is still running." % (onstart_handler.__name__, e))

    def stop(self):
        """Stops the pastepwn instance"""
        self.scraping_handler.stop()
        self.paste_dispatcher.stop()
        self.action_handler.stop()
        self.is_idle = False

    def signal_handler(self, signum, frame):
        """Handler method to handle signals"""
        self.is_idle = False
        self.logger.info("Received signal {}, stopping...".format(signum))
        self.stop()

    def idle(self, stop_signals=(SIGINT, SIGTERM, SIGABRT)):
        """Blocks until one of the signals are received and stops the updater.
        Thanks to the python-telegram-bot developers - https://github.com/python-telegram-bot/python-telegram-bot/blob/2cde878d1e5e0bb552aaf41d5ab5df695ec4addb/telegram/ext/updater.py#L514-L529
        :param stop_signals: The signals to which the code reacts to
        """
        self.is_idle = True
        self.logger.info("In Idle!")

        for sig in stop_signals:
            signal(sig, self.signal_handler)

        while self.is_idle:
            if self.__exception_event.is_set():
                self.logger.warning("An exception occurred. Calling exception handlers and going down!")
                for handler in self.error_handlers:
                    # call the error handlers in case of an exception
                    handler()
                self.is_idle = False
                self.stop()
                return

            sleep(1)



# -*- coding: utf-8 -*-

from .abstractdb import AbstractDB
from .mongodb import MongoDB
from .mysqldb import MysqlDB
from .sqlitedb import SQLiteDB

__all__ = ["AbstractDB", "MongoDB", "SQLiteDB", "MysqlDB"]



# -*- coding: utf-8 -*-
import unittest

from pastepwn.scraping.pastebin import PastebinScraper
from pastepwn.scraping.pastebin.exceptions import IPNotRegisteredError, PasteDeletedException, PasteNotReadyException, PasteEmptyException


class TestPastebinscraper(unittest.TestCase):

    def setUp(self) -> None:
        self.pastebinscraper = PastebinScraper()

    def test_empty(self):
        with self.assertRaises(PasteEmptyException):
            self.pastebinscraper._check_error("")

    def test_not_ready(self):
        with self.assertRaises(PasteNotReadyException):
            self.pastebinscraper._check_error("File is not ready for scraping yet. Try again in 1 minute.")

    def test_deleted(self):
        with self.assertRaises(PasteDeletedException):
            self.pastebinscraper._check_error("Error, we cannot find this paste.")

    def _check_ip_not_registered(self, ip_list):
        shell = "YOUR IP: {} DOES NOT HAVE ACCESS. VISIT: https://pastebin.com/doc_scraping_api TO GET ACCESS!"
        for ip in ip_list:
            with self.assertRaises(IPNotRegisteredError):
                self.pastebinscraper._check_error(shell.format(ip))
                print("The following IP was not detected: {}".format(ip))

    def test_ipv4_not_registered(self):
        """Test if the _check_error method detects different IPv4 addresses. It's okay to also detect invalid addresses where an octed is > 255)"""
        ipv4_test = ["1.1.1.1", "10.1.5.6", "1.10.5.6", "1.1.50.6", "1.1.5.60", "1.1.50.60", "1.10.50.60", "10.10.50.60", "10.10.50.255", "10.10.255.255",
                     "10.255.255.255", "255.255.255.255", "333.333.333.333"]

        self._check_ip_not_registered(ipv4_test)

    def test_ipv6_not_registered(self):
        ipv6_test = ["fe80::21d8:f50:c295:c4be", "2001:cdba:0000:0000:0000:0000:3257:9652", "2001:cdba:0:0:0:0:3257:9652", "2001:cdba::3257:9652",
                     "2001:cdba::1222", "21DA:D3:0:2F3B:2AA:FF:FE28:9C5A", "2001:cdba::1:2:3:3257:9652", "FE80::8329", "FE80::FFFF:8329",
                     "FE80::B3FF:FFFF:8329", "FE80::0202:B3FF:FFFF:8329", "FE80:0000:0000:0000:0202:B3FF:FFFF:8329"]
        # TODO: IPv6 addresses with double colon AND full zero groups (of 16 bits) are currently not recognized by the used regex. An example address would
        #  be: `FE80::0000:0000:0202:B3FF:FFFF:8329`

        self._check_ip_not_registered(ipv6_test)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import re

from .basicanalyzer import BasicAnalyzer


class RegexAnalyzer(BasicAnalyzer):
    """Analyzer to match the content of a paste via regular expressions"""
    name = "RegexAnalyzer"

    def __init__(self, actions, regex, flags=0):
        super().__init__(actions)
        self.regex = re.compile(regex, flags)

    def verify(self, results):
        """Method to perform additional checks to test if the matches are actually valid"""
        # This method can be overwritten by subclasses in order to perform more checks within the analyzer
        return results

    def match(self, paste):
        """Match the content of a paste via regex. Return true if regex matches"""
        if paste is None:
            return False

        paste_content = paste.body or ""
        matches = self.regex.findall(paste_content)

        return self.verify(matches)



# -*- coding: utf-8 -*-
import unittest
from unittest.mock import Mock
import re

from pastepwn.analyzers.regexanalyzer import RegexAnalyzer


class TestRegexAnalyzer(unittest.TestCase):

    def setUp(self):
        """Set's up a paste mock object"""
        self.paste = Mock()

    def test_matchWord(self):
        analyzer = RegexAnalyzer(None, regex="word")
        self.paste.body = "This text contains the word 'word'!"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match, although it should!")
        self.assertEqual("word", analyzer.match(self.paste)[0])
        self.assertEqual("word", analyzer.match(self.paste)[1])
        self.assertEqual(2, len(analyzer.match(self.paste)))

    def test_matchPattern(self):
        analyzer = RegexAnalyzer(None, regex=r"\d{4}-123-\d{2}")
        self.paste.body = "This text contains 4444-123-55 a nice pattern!"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match, although it should!")
        self.assertEqual("4444-123-55", analyzer.match(self.paste)[0])

        self.paste.body = "This text contains 4657-123-57 a nice pattern!"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match, although it should!")
        self.assertEqual("4657-123-57", analyzer.match(self.paste)[0])

        self.paste.body = "This text contains 465-123-57 a bad pattern!"
        self.assertFalse(analyzer.match(self.paste), "The regex does match, although it shouldn't!")
        self.assertEqual([], analyzer.match(self.paste))

    def test_multiple(self):
        analyzer = RegexAnalyzer(None, regex=r"\d{4}-123-\d{2}")
        self.paste.body = "This text contains 4444-123-55 and 1864-123-11 and 1649-123-00 which in total are three nice patterns!"
        self.assertEqual(3, len(analyzer.match(self.paste)))
        self.assertEqual("4444-123-55", analyzer.match(self.paste)[0])
        self.assertEqual("1864-123-11", analyzer.match(self.paste)[1])
        self.assertEqual("1649-123-00", analyzer.match(self.paste)[2])

    def test_flags(self):
        """We only test a few flags, because as long as we use regex this should work fine"""
        analyzer = RegexAnalyzer(None, regex=r"ThIs iS mIxEd")
        self.paste.body = "this is mixed case (not)"
        self.assertFalse(analyzer.match(self.paste), "The regex does match, although it shouldn't!")

        # Test case independend flag / ignorecase
        analyzer = RegexAnalyzer(None, regex=r"ThIs iS mIxEd", flags=re.IGNORECASE)
        self.paste.body = "this is mixed case (not)"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match, although it should!")

        # Test multiline
        analyzer = RegexAnalyzer(None, regex=r"^regexiscool", flags=re.MULTILINE)
        self.paste.body = "this is a multiline string\nregexiscool is right at the start of the line\nand another line!"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match in multiline strings, although it should!")

    def test_negativeMatch(self):
        """Test a case where the regex should not match"""
        analyzer = RegexAnalyzer(None, regex=r"[a-zA-Z]{15}")
        self.paste.body = "abcdef1ghijklmn and a lot more characters"
        self.assertFalse(analyzer.match(self.paste), "The regex does match, although it shouldn't!")

    def test_emptyBody(self):
        """Empty bodies should not match"""
        analyzer = RegexAnalyzer(None, regex=r"[0-9]", flags=re.MULTILINE)
        self.paste.body = ""
        self.assertFalse(analyzer.match(self.paste), "The regex does match, although it shouldn't!")
        self.assertEqual([], analyzer.match(self.paste))

    def test_verify(self):
        """Check that the verify method gets called correctly"""

        def _verify_method(matches):
            self.test = ""
            return matches

        class TestAnalyzer(RegexAnalyzer):
            """Test analyzer for testing the verify method"""
            verify = Mock(side_effect=_verify_method)

            def __init__(self, actions, regex):
                super().__init__(actions, regex)

        mock_paste = Mock()
        mock_paste.body = "Test aBc Test"
        t = TestAnalyzer([], r"aBc")
        result = t.match(mock_paste)
        t.verify.assert_called_with(["aBc"])

        self.assertTrue(result)
        self.assertEqual(1, len(result))
        self.assertEqual("aBc", result[0])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.databasedumpanalyzer import DatabaseDumpAnalyzer


class TestDatabaseDumpAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = DatabaseDumpAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        self.paste.body = """(`id`, `team_id`, `email`, `name`, `password`, `league`, `active`, `regdate`, `lan`, `lastlogin`, `birthdate`, `favclub`, `favmanager`, `description`, `pers_email`, `mess_id`, `iso`)
                             (14, 568, 'vcpd_@hotmail.com', 'Flavio00', '059b4db7cdb1cbddc3f0e5d95c881597', 1, 1, 1224313200, 0, 0, 0, '', '', '', '', '', ''),
                             (4, 1, 'levi@medeeaweb.com', 'Slash', 'c57aeddaffce62fead6be61022eb1340', 1, 1, 1224313200, 0, 1235380637, 483260400,
                             'FC Juventus Torino', 'Carlo Ancelotti', 'I''m the admin of this site :D', 'slash@manager-arena.com', 'slashwebdesign', ''),"""
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(1, len(match))
        self.assertEqual("""(`id`, `team_id`, `email`, `name`, `password`, `league`, `active`, `regdate`, `lan`, `lastlogin`, `birthdate`, `favclub`, `favmanager`, `description`, `pers_email`, `mess_id`, `iso`)""", match[0])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import json
import logging
import re
import time
from queue import Queue, Empty

from pastepwn.core import Paste
from pastepwn.scraping import BasicScraper
from pastepwn.scraping.pastebin.exceptions import IPNotRegisteredError, PasteDeletedException, PasteNotReadyException, PasteEmptyException
from pastepwn.util import Request, start_thread

# https://pastebin.com/doc_scraping_api#2
# Your whitelisted IP should not run into any issues as long as you don't abuse our service.
# We recommend not making more than 1 request per second, as there really is no need to do so.
# Going over 1 request per second won't get you blocked, but if we see excessive unnecessary scraping, we might take action.


class PastebinScraper(BasicScraper):
    """Scraper class for pastebin"""
    name = "PastebinScraper"
    api_base_url = "https://scrape.pastebin.com"
    pastebin_error_pattern = re.compile(r"^YOUR IP: ((\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})|((([0-9A-Fa-f]{1,4}:){7})([0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,6}:)(([0-9A-Fa-f]{1,4}:){0,4})([0-9A-Fa-f]{1,4}))) DOES NOT HAVE ACCESS\.\s+VISIT: https://pastebin\.com/doc_scraping_api TO GET ACCESS!")

    def __init__(self, paste_queue=None, exception_event=None, api_hit_rate=None):
        super().__init__(exception_event)
        self.logger = logging.getLogger(__name__)
        self._last_scrape_time = 0
        self.paste_queue = paste_queue or Queue()
        self._tmp_paste_queue = Queue()

        self._known_pastes = []
        self._known_pastes_limit = 1000

        # The hit rate describes the interval between two requests in seconds
        self._api_hit_rate = api_hit_rate or 1

    def _check_error(self, body, key=None):
        """Checks if an error occurred and raises an exception if it did"""
        if self.pastebin_error_pattern.match(body):
            self._exception_event.set()
            raise IPNotRegisteredError(body)

        if body is None or body == "":
            raise PasteEmptyException("The paste '{0}' or its body was set to None!".format(key))
        if body == "File is not ready for scraping yet. Try again in 1 minute.":
            # The pastebin API was not ready yet to deliver this paste -
            # We raise an exception to re-download it again after some time has passed
            raise PasteNotReadyException("The paste '{0}' could not be fetched yet!".format(key))
        elif body == "Error, we cannot find this paste.":
            # The paste has been deleted before we could download it.
            # We raise an exception to delete the paste from the queue
            raise PasteDeletedException("The paste '{0}' has been deleted!".format(key))

    def _get_recent(self, limit=100):
        """Downloads a list of the most recent pastes - the amount is limited by the <limit> parameter"""
        r = Request()
        endpoint = "api_scraping.php"
        api_url = "{0}/{1}?limit={2}".format(self.api_base_url, endpoint, limit)

        try:
            response_data = r.get(api_url)

            self._check_error(response_data)

            pastes_dict = json.loads(response_data)
            pastes = []

            # Loop through the response and create objects by the data
            for paste in pastes_dict:
                paste_obj = Paste(key=paste.get("key"),
                                  title=paste.get("title"),
                                  user=paste.get("user"),
                                  size=paste.get("size"),
                                  date=paste.get("date"),
                                  expire=paste.get("expire"),
                                  syntax=paste.get("syntax"),
                                  scrape_url=paste.get("scrape_url"),
                                  full_url=paste.get("full_url")
                                  )
                pastes.append(paste_obj)

            return pastes
        except Exception as e:
            self.logger.error(e)
            return None

    def _get_paste_content(self, key):
        """Downloads the content of a certain paste"""
        r = Request()
        endpoint = "api_scrape_item.php"
        api_url = "{0}/{1}?i={2}".format(self.api_base_url, endpoint, key)

        self.logger.debug("Downloading paste {0}".format(key))
        try:
            response_data = r.get(api_url)
        except Exception as e:
            self.logger.error(e)
            raise e

        self._check_error(response_data, key)

        return response_data

    def _body_downloader(self):
        """Downloads the body of pastes from pastebin, which have been put into the queue"""
        while self.running:
            # Print current approx. size of paste queue
            if self._tmp_paste_queue.qsize() > 0:
                self.logger.debug("Queue size: {}".format(self._tmp_paste_queue.qsize()))

            # Check if the stop event or exception events are set
            if self._stop_event.is_set() or self._exception_event.is_set():
                self.logger.debug("Stop or exception event is set!")
                self.running = False
                break

            try:
                paste = self._tmp_paste_queue.get(True, 1)
            except Empty:
                continue

            last_body_download_time = round(time.time(), 2)

            try:
                body = self._get_paste_content(paste.key)
            except PasteNotReadyException:
                self.logger.debug("Paste '{0}' is not ready for downloading yet. Enqueuing it again.".format(paste.key))
                # Make sure to wait a certain time. If only one element in the queue, this can lead to loops
                self._rate_limit_sleep(last_body_download_time)
                self._tmp_paste_queue.put(paste)
                continue
            except PasteDeletedException:
                # We don't add a sleep here, because this can't lead to loops
                self.logger.info("Paste '{0}' has been deleted before we could download it! Skipping paste.".format(paste.key))
                continue
            except PasteEmptyException:
                self.logger.info("Paste '{0}' is set to None! Skipping paste.".format(paste.key))
                continue
            except Exception as e:
                self.logger.error("An exception occurred while downloading the paste '{0}'. Skipping this paste! Exception is: {1}".format(paste.key, e))
                continue

            paste.set_body(body)
            self.paste_queue.put(paste)

            self._rate_limit_sleep(last_body_download_time)

    def _rate_limit_sleep(self, last_body_download_time):
        """Sleeps a certain amount of seconds to prevent hitting API rate limits
        :param last_body_download_time: The time when the last paste was downloaded
        :return:
        """
        current_time = round(time.time(), 2)
        diff = round(current_time - last_body_download_time, 2)

        if diff >= self._api_hit_rate:
            return

        sleep_diff = round(self._api_hit_rate - diff, 3)
        self.logger.debug("Sleep time is: {0}".format(sleep_diff))
        time.sleep(sleep_diff)

    def start(self, paste_queue):
        """Start the scraping process and download the paste metadata"""
        self.paste_queue = paste_queue
        self.running = True
        start_thread(self._body_downloader, "BodyDownloader", self._exception_event)

        while self.running:
            self._last_scrape_time = int(time.time())
            pastes = self._get_recent(limit=100)
            counter = 0

            if pastes is not None:
                for paste in pastes:
                    # check if paste is in list of known pastes
                    if paste.key in self._known_pastes:
                        # Do nothing, if it's already known
                        continue

                    self.logger.debug("Paste is unknown - adding ot to list {}".format(paste.key))
                    self._tmp_paste_queue.put(paste)
                    self._known_pastes.append(paste.key)
                    counter += 1

                    if self._stop_event.is_set() or self._exception_event.is_set():
                        self.running = False
                        break

                self.logger.debug("{0} new pastes fetched!".format(counter))

            # Delete some of the last pastes to not run into memory/performance issues
            if len(self._known_pastes) > 1000:
                self.logger.debug("known_pastes > 1000 - cleaning up!")
                start_index = len(self._known_pastes) - self._known_pastes_limit
                self._known_pastes = self._known_pastes[start_index:]

            if self._stop_event.is_set() or self._exception_event.is_set():
                self.logger.debug("stopping {0}".format(self.name))
                self.running = False
                break

            while self.running:
                current_time = int(time.time())
                diff = current_time - self._last_scrape_time

                if diff > 60:
                    break

                # if the last scraping happened less than 60 seconds ago,
                # wait 2 seconds and check again
                time.sleep(2)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class IPv4AddressAnalyzer(RegexAnalyzer):
    """Analyzer to match on ip addresses via regex"""
    name = "IPv4AddressAnalyzer"

    def __init__(self, actions):
        regex = r"\b\d{1,3}(?:\.\d{1,3}){3}\b"
        super().__init__(actions, regex)

    def verify(self, results):
        """Verify results to get only real IP adresses"""
        verified_ips = []
        for result in results:
            # Check each IP
            octet_max_value = 255
            for octet in result.split("."):
                if int(octet) > octet_max_value:
                    break
            else:
                # IP is valid
                verified_ips.append(result)

        return verified_ips



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.logicalanalyzers import OrAnalyzer


class TestOrAnalyzer(unittest.TestCase):
    def setUp(self):
        self.paste = mock.Mock()
        self.gt = mock.Mock()
        self.gt.match = mock.Mock(return_value=True)

        self.gf = mock.Mock()
        self.gf.match = mock.Mock(return_value=False)

    def test_match_positive(self):
        self.paste.body = "Test"
        analyzer = OrAnalyzer(None, [self.gt])
        self.assertTrue(analyzer.match(self.paste))

        analyzer = OrAnalyzer(None, [self.gt, self.gt, self.gt])
        self.assertTrue(analyzer.match(self.paste))

        analyzer = OrAnalyzer([], [self.gf, self.gf])
        self.assertFalse(analyzer.match(self.paste))

        analyzer = OrAnalyzer([], [self.gf, self.gf, self.gf])
        self.assertFalse(analyzer.match(self.paste))

        analyzer = OrAnalyzer([], [self.gf, self.gf, self.gf, self.gt])
        self.assertTrue(analyzer.match(self.paste))

    def test_negative(self):
        self.paste.body = ""

        analyzer = OrAnalyzer([], None)
        self.assertFalse(analyzer.match(self.paste))

        analyzer = OrAnalyzer([], [])
        self.assertFalse(analyzer.match(self.paste))

    def test_actions_present(self):
        action = mock.MagicMock(spec=BasicAction)
        analyzer = OrAnalyzer(action, None)
        self.assertEqual([action], analyzer.actions)

    def test_analyzers_present(self):
        analyzer = OrAnalyzer(None, self.paste)
        self.assertEqual([self.paste], analyzer.analyzers)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import json

from .savefileaction import SaveFileAction


class SaveJSONAction(SaveFileAction):
    """Action to save a json formatted file to the disk"""
    name = "SaveJSONAction"

    def __init__(self, path):
        super().__init__(path, file_ending=".json")

    def get_file_content(self, paste, analyzer_name, matches):
        """Returns the content to be written to the file"""
        return json.dumps(paste.to_dict())



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.basicanalyzer import BasicAnalyzer
from pastepwn.analyzers.basicanalyzer import MergedAnalyzer


class TestMergedAnalyzer(unittest.TestCase):
    class NewAnalyzer(BasicAnalyzer):
        """Test Analyzer for testing mergedAnalyzer"""

        def __init__(self, return_value):
            super().__init__(actions=None)
            self.return_value = return_value

        def match(self, paste):
            """Match func"""
            return self.return_value

    def setUp(self):
        """Setup test case"""
        self.true_analyzer = self.NewAnalyzer(True)
        self.false_analyzer = self.NewAnalyzer(False)
        self.paste_mock = mock.Mock()
        self.paste_mock.body = "This is a mock paste"

    def test_and(self):
        """Check if logical and between analyzers works fine"""
        and_analyzer = self.true_analyzer & self.false_analyzer

        # One analyzer returns False, the other True, this should evaluate to False
        self.assertFalse(and_analyzer.match(self.paste_mock))

        and_analyzer2 = self.true_analyzer & self.true_analyzer
        # Since both analyzers return true, this should now return True as well
        self.assertTrue(and_analyzer2.match(self.paste_mock))

    def test_or(self):
        """Check if logical or between analyzers works fine"""
        # One analyzer returns False, the other True, this should evaluate to True
        or_analyzer = self.true_analyzer | self.false_analyzer
        self.assertTrue(or_analyzer.match(self.paste_mock))

        # Since both return true, this should return True as well
        or_analyzer2 = self.true_analyzer | self.false_analyzer
        self.assertTrue(or_analyzer2.match(self.paste_mock))

        or_analyzer3 = self.false_analyzer | self.true_analyzer
        self.assertTrue(or_analyzer3.match(self.paste_mock))

    def test_both(self):
        """Check if logical and/or both work fine in combination with each other"""
        # One analyzer returns False, the other True, this should evaluate to False
        and_analyzer = self.true_analyzer & self.false_analyzer
        self.assertFalse(and_analyzer.match(self.paste_mock))

        # Since both return true, this should return True as well
        or_analyzer = self.true_analyzer | self.false_analyzer
        self.assertTrue(or_analyzer.match(self.paste_mock))

    def test_none(self):
        """Check that error is raised in case no value is given for and/or/not_analyzer"""
        with self.assertRaises(ValueError):
            MergedAnalyzer(base_analyzer=None)

    def test_long_chain(self):
        """Check if logical and/or both work fine in long combinations with each other"""
        # Long chain of true_analyzers must evaluate to True
        and_analyzer = self.true_analyzer & self.true_analyzer & self.true_analyzer & self.true_analyzer & \
            self.true_analyzer & self.true_analyzer & self.true_analyzer & self.true_analyzer
        self.assertTrue(and_analyzer.match(self.paste_mock))

        # A single false_analyzer must make the term evaluate to false
        and_analyzer2 = self.true_analyzer & self.true_analyzer & self.true_analyzer & self.false_analyzer & \
            self.true_analyzer & self.true_analyzer & self.true_analyzer & self.true_analyzer
        self.assertFalse(and_analyzer2.match(self.paste_mock))

        # Since one returns true, this should return True as well
        or_analyzer = self.false_analyzer | self.false_analyzer | self.false_analyzer | self.false_analyzer | \
            self.false_analyzer | self.true_analyzer | self.true_analyzer | self.true_analyzer
        self.assertTrue(or_analyzer.match(self.paste_mock))

    def test_list_and(self):
        """Check if other return values than booleans are handled correctly with logical and"""
        and_analyzer = self.NewAnalyzer(["Test", 123]) & self.NewAnalyzer([])
        res = and_analyzer.match(self.paste_mock)
        self.assertIsInstance(res, bool)
        self.assertFalse(res)

    def test_list_or(self):
        """Check if other return values than booleans are handled correctly with logical or"""
        and_analyzer = self.NewAnalyzer(["Test", 123]) | self.NewAnalyzer([])
        res = and_analyzer.match(self.paste_mock)
        self.assertIsInstance(res, bool)
        self.assertTrue(res)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Do not mess with the order of the imports
# Otherwise there will be circular imports -> bad

from .core.pastepwn import PastePwn
from .core.paste import Paste
from .core.pastedispatcher import PasteDispatcher
from .core.scrapinghandler import ScrapingHandler
from .core.actionhandler import ActionHandler

__author__ = "d-Rickyy-b (pastepwn@rico-j.de)"

__all__ = ["PastePwn", "Paste", "PasteDispatcher", "ScrapingHandler", "ActionHandler"]



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.emailpasswordpairanalyzer import EmailPasswordPairAnalyzer


class TestEmailPasswordPairAnalyzer(unittest.TestCase):
    def setUp(self):
        self.paste = mock.Mock()

    def test_positive(self):
        """Test single matches in a paste"""
        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "estocanam2@gmail.com:Firebird1@"
        self.assertTrue(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "test+test@gmail.com:abcd"
        self.assertTrue(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "estocanam2@gmail.com:aq12ws"
        self.assertTrue(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "estocanam2@apple.com:Fireb§"
        self.assertTrue(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "g@bb.com:Firebird1@"
        self.assertTrue(analyzer.match(self.paste))

    def test_negative(self):
        """Tests if it does not match on wrong strings"""
        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "This is a Test"
        self.assertFalse(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "{a: 'b'}"
        self.assertFalse(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = ""
        self.assertFalse(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "\t\n"
        self.assertFalse(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "\n\n"
        self.assertFalse(analyzer.match(self.paste))

    def test_match_multiple(self):
        """Test multiple matches in a single paste"""
        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "estocanam2@apple.com:Fireb§\n" \
                          "g@bb.com:Firebird1@\n" \
                          "Some comment\n" \
                          "test+test@gmail.com:abcd"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("estocanam2@apple.com:Fireb§", match[0])
        self.assertEqual("g@bb.com:Firebird1@", match[1])
        self.assertEqual("test+test@gmail.com:abcd", match[2])

    def test_min_match(self):
        """Test if the setting for minimal matches works"""
        analyzer = EmailPasswordPairAnalyzer(None, min_amount=1)
        self.paste.body = "estocanam2@apple.com:Fireb§\n" \
                          "g@bb.com:Firebird1@\n" \
                          "Some comment\n" \
                          "test+test@gmail.com:abcd"
        self.assertEqual(3, len(analyzer.match(self.paste)))
        self.assertTrue(analyzer.match(self.paste))

        analyzer = EmailPasswordPairAnalyzer(None, min_amount=3)
        self.assertTrue(analyzer.match(self.paste))
        self.assertEqual(3, len(analyzer.match(self.paste)))

        analyzer = EmailPasswordPairAnalyzer(None, min_amount=4)
        self.assertFalse(analyzer.match(self.paste))
        self.assertEqual(bool, type(analyzer.match(self.paste)))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "An email/pw combo estocanam2@apple.com:Fireb§ inside a text"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("estocanam2@apple.com:Fireb§", match[0])

        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "also middle inside\nof test+test@gmail.com:abcd of a string!"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("test+test@gmail.com:abcd", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        analyzer = EmailPasswordPairAnalyzer(None)
        self.paste.body = "An email/pw combo estocanam2@apple.com:Fireb§ and also another inside\nof test+test@gmail.com:abcd of a string!"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("estocanam2@apple.com:Fireb§", match[0])
        self.assertEqual("test+test@gmail.com:abcd", match[1])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class AWSSessionTokenAnalyzer(RegexAnalyzer):
    """Analyzer to match AWS Session Token via regex."""
    name = "AWSSessionTokenAnalyzer"

    def __init__(self, actions):
        regex = r"(?:(?:\\\"|'|`)?(?:i?aws)?_?(?:i?session)?_?(?:i?token)?(?:\\\"|'|`)?\\\\s{0,50}(?::|=>|=)\\\\s{0,50}(?:\\\"|'|`)?[A-Za-z0-9/+=]{16,}(?:\\\"|'|`)?)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import re
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from pastepwn.util import TemplatingEngine
from .basicaction import BasicAction


class EmailAction(BasicAction):
    """This action sends out an e-mail to the receiver containing the paste, when executed"""
    name = "EmailAction"

    def __init__(self, username, password, receiver, hostname, port=465, template=None):
        super().__init__()
        mail_regex = r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)"
        if username is None or not re.match(mail_regex, username):
            raise ValueError("Invalid username !")
        else:
            self.username = username
        if receiver is None or not re.match(mail_regex, receiver):
            raise ValueError("Invalid reciever address !")
        else:
            self.receiver = receiver
        self.password = password
        self.hostname = hostname
        self.port = port
        self.template = template

    def perform(self, paste, analyzer_name=None, matches=None):
        """Sends an email to the specified receiver with the paste's content.
        :param paste: The paste passed by the ActionHandler
        :param analyzer_name: The name of the analyzer which matched the paste
        :param matches: A list of matches, on which the analyzer matched on
        :return: None
        """
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)

        email = MIMEMultipart()
        email["From"] = self.username
        email["To"] = self.receiver
        email["Subject"] = "Paste matched by pastepwn via analyzer '{}'".format(analyzer_name)
        email.attach(MIMEText(text, "plain"))

        # TODO there should be a way to use starttls - check https://realpython.com/python-send-email/
        with smtplib.SMTP_SSL(self.hostname, self.port) as smtp:
            smtp.login(self.username, self.password)
            text = email.as_string()
            smtp.sendmail(self.username, self.receiver, text)



# -*- coding: utf-8 -*-
import re
import unittest
from unittest import mock

from pastepwn.analyzers.pastetitleanalyzer import PasteTitleAnalyzer


class TestPasteTitleAnalyzer(unittest.TestCase):
    def setUp(self):
        """Set's up a paste mock object"""
        self.paste = mock.Mock()

    def test_matchWord(self):
        analyzer = PasteTitleAnalyzer(None, regex=r"word")
        self.paste.title = "This text contains the word 'word'!"
        self.assertTrue(analyzer.match(self.paste), "Title does not match, although it should!")

    def test_matchPattern(self):
        analyzer = PasteTitleAnalyzer(None, regex=r"\d{2}-\d{2}-\d{4}")
        self.paste.title = "This is a date 24-01-2020 inside the title"
        self.assertTrue(analyzer.match(self.paste), "Title does not match pattern!")

    def test_matchPattern2(self):
        analyzer = PasteTitleAnalyzer(None, regex=r"^The title")
        self.paste.title = "The title is cool!"
        self.assertTrue(analyzer.match(self.paste), "Title does not match pattern!")

    def test_flags(self):
        """We only test a few flags, because as long as we use regex this should work fine"""
        analyzer = PasteTitleAnalyzer(None, regex=r"ThIs iS mIxEd")
        self.paste.title = "this is mixed case (not)"
        self.assertFalse(analyzer.match(self.paste), "The regex does match, although it shouldn't!")

        # Test case independend flag / ignorecase
        analyzer = PasteTitleAnalyzer(None, regex=r"ThIs iS mIxEd", flags=re.IGNORECASE)
        self.paste.title = "this is mixed case (not)"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match, although it should!")

        # Test multiline
        analyzer = PasteTitleAnalyzer(None, regex=r"^regexiscool", flags=re.MULTILINE)
        self.paste.title = "this is a multiline string\nregexiscool is right at the start of the line\nand another line!"
        self.assertTrue(analyzer.match(self.paste), "The regex does not match in multiline strings, although it should!")

    def test_emptyTitle(self):
        analyzer = PasteTitleAnalyzer(None, regex=r"Any \d pattern")
        self.paste.title = ""
        self.assertFalse(analyzer.match(self.paste), "Empty title does match pattern, although it  shouldn't!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class AWSSecretKeyAnalyzer(RegexAnalyzer):
    """
    Analyzer to match AWS Secret Key via regex

    Keys are 40 character alphanumeric with a few symbols /+=
    """
    name = "AWSSecretKeyAnalyzer"

    def __init__(self, actions):
        regex = r"\b[A-Za-z0-9/+=]{40}\b"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-


class PastepwnError(Exception):
    """Representation of a pastepwn error object."""
    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class InvalidActionError(PastepwnError):
    """Representation of an error for invalid actions passed to analyzers"""
    pass



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class EpicKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match Epic Licensing Keys"""

    def __init__(self, action):
        """
        Analyzer to match Epic Licensing Keys
        :param action: Single action or list of actions to be executed when a paste matches
        """
        # Applied general A-Z or 0-9 based on example provided
        # Regex can be adjusted if certain characters are not valid
        regex = r"\b(?<!-)[A-Z0-9]{5}\-[A-Z0-9]{5}\-[A-Z0-9]{5}\-[A-Z0-9]{5}\b(?!-)"

        super().__init__(action, regex)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.exactwordanalyzer import ExactWordAnalyzer


class TestExactWordAnalyzer(unittest.TestCase):
    def setUp(self):
        self.paste = mock.Mock()

    def test_match(self):
        analyzer = ExactWordAnalyzer(None, "Test")
        self.paste.body = "This is a Test"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "There is a test for everything"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "There are tests for everything"
        self.assertFalse(analyzer.match(self.paste))

        self.paste.body = "This is a Test for a longer sentence"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "Completely unrelated"
        self.assertFalse(analyzer.match(self.paste))

    def test_blacklist(self):
        blacklist = ["fake", "bad"]
        analyzer = ExactWordAnalyzer(None, "Test", blacklist=blacklist)

        self.paste.body = "This is a Test"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "This is a fake Test"
        self.assertFalse(analyzer.match(self.paste))

        analyzer = ExactWordAnalyzer(None, "Test", blacklist=blacklist, case_sensitive=True)

        self.paste.body = "This is a Test"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "This is a Fake Test"
        self.assertTrue(analyzer.match(self.paste))

    def test_multiple_words(self):
        analyzer = ExactWordAnalyzer(None, None)
        self.assertEqual(analyzer.words, [])

        analyzer = ExactWordAnalyzer(None, ["My", "first", "Test"])
        self.paste.body = "This is a little test for something"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "You are my best friend so far!"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "This is the first time I try this"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "This time we try to match multiple words/tests for the first time."
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "None of the words are contained!"
        self.assertFalse(analyzer.match(self.paste))

        # Check for case sensitivity for multiple words
        analyzer2 = ExactWordAnalyzer(None, ["My", "first", "Test"], case_sensitive=True)

        self.paste.body = "That's not my issue!"
        self.assertFalse(analyzer2.match(self.paste))

        self.paste.body = "That's not My issue!"
        self.assertTrue(analyzer2.match(self.paste))

    def test_add_word(self):
        analyzer = ExactWordAnalyzer(None, "Test")
        self.assertEqual(len(analyzer.words), 1)
        self.assertEqual(analyzer.words, ["Test"])

        analyzer.add_word("second")
        self.assertEqual(len(analyzer.words), 2)
        self.assertEqual(analyzer.words, ["Test", "second"])

    def test_case_sensitive(self):
        analyzer = ExactWordAnalyzer(None, "Test", case_sensitive=True)
        self.paste.body = "This is a Test for case sensitivity"
        self.assertTrue(analyzer.match(self.paste))

        self.paste.body = "This is a test for case sensitivity"
        self.assertFalse(analyzer.match(self.paste))

        self.paste.body = "This is a tESt for case sensitivity"
        self.assertFalse(analyzer.match(self.paste))

        analyzer2 = ExactWordAnalyzer(None, "Te1st")
        self.paste.body = "This is a te1st for case sensitivity"
        self.assertTrue(analyzer2.match(self.paste))

        analyzer2 = ExactWordAnalyzer(None, "Te1st")
        self.paste.body = "This is a tE1st for case sensitivity"
        self.assertTrue(analyzer2.match(self.paste))

    def test_multiple_case_sensitive(self):
        """Test if it's possible to match any of multiple words in a wordanalyzer when case sensitivty is activated"""
        analyzer = ExactWordAnalyzer(None, ["My", "first", "Test"], case_sensitive=True)
        self.paste.body = "This is a little test for something"
        self.assertFalse(analyzer.match(self.paste))

        self.paste.body = "You are my best friend so far!"
        self.assertFalse(analyzer.match(self.paste))

        self.paste.body = "Myself and I"
        self.assertFalse(analyzer.match(self.paste))

        self.paste.body = "This is a Test for case sensitivity"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(1, len(match))
        self.assertEqual("Test", match[0])

        self.paste.body = "This is a test for case sensitivity. It's the first of its kind."
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(1, len(match))
        self.assertEqual("first", match[0])

    def test_match_none(self):
        analyzer = ExactWordAnalyzer(None, "Test")
        self.paste.body = None
        self.assertFalse(analyzer.match(self.paste))

        self.paste = None
        self.assertFalse(analyzer.match(self.paste))

    def test_match_empty(self):
        analyzer = ExactWordAnalyzer(None, "Test")
        self.paste.body = ""
        self.assertFalse(analyzer.match(self.paste))

    def test_actions_present(self):
        action = mock.MagicMock(spec=BasicAction)
        analyzer = ExactWordAnalyzer(action, "Test")
        self.assertEqual([action], analyzer.actions)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class PhoneNumberAnalyzer(RegexAnalyzer):
    """Analyzer to match Phone Numbers"""

    def __init__(self, action):
        """Analyzer to match international phone numbers"""
        # from stackoverflow: https://stackoverflow.com/questions/2113908/what-regular-expression-will-match-valid-international-phone-numbers
        regex = r"\+(?:9[976]\d|8[987530]\d|6[987]\d|5[90]\d|42\d|3[875]\d|2[98654321]\d|9[8543210]|8[6421]|6[6543210]|5[87654321]|4[987654310]|3[" \
                r"9643210]|2[70]|7|1)[\d \-\(\)]{2,16}(?!\d)"

        super().__init__(action, regex)

    def verify(self, results):
        """Verify possible phone numbers"""
        # Currently we don't really do verifications - we only replace spaces in the end,
        # because our regex also matches whitespace in the end
        return [result.rstrip() for result in results]



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class Base64Analyzer(RegexAnalyzer):
    """Analyzer to match base64 encoding via regex"""
    name = "Base64Analyzer"

    def __init__(self, actions, min_len=1):
        regex = r"(?<![A-Za-z0-9+\/=])(?:[A-Za-z0-9+\/]{4})+(?:[A-Za-z0-9+\/]{2}==|[A-Za-z0-9+\/]{3}=)?(?![A-Za-z0-9+\/=])"
        self.min_len = min_len
        super().__init__(actions, regex)

    def verify(self, results):
        """Method to perform additional checks to test if the found strings are of sufficient length"""
        validated_strings = []

        for result in results:
            if len(result) >= self.min_len:
                validated_strings.append(result)

        return validated_strings



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.awssessiontokenanalyzer import AWSSessionTokenAnalyzer


class TestAWSSessionTokenAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = AWSSessionTokenAnalyzer(None)
        self.paste = mock.Mock()

    def test_mach_positive(self):
        """Test if positives are recognized"""
        self.paste.body = r"'aws_session_token'\\ssss:\\ssss'AiughaiusDWIHJFUFERHO2134234'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"'aws'\\ssss:\\ssss'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session'\\ssss:\\ssss'YTUF5GUY76ibuihIUIU98jJB+//='"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session_token'\\s:\\s'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session_token'\\:\\'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session_token'\\:\\'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"\\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(r"\\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'", match[0])

        self.paste.body = r"\\=\\'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = r"\\=>\\'auyhguywgerdbyubduiywebh'"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = r"Please always use this session token: \\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'. Cu soon."
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(r"\\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'", match[0])

        self.paste.body = r"Also there are other tokens such as \\=\\'auyhguywgerdbyubduiywebh' which is pretty short"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(r"\\=\\'auyhguywgerdbyubduiywebh'", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = r"Please always use this session token: " \
                          r"\\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'. Also we can use shorter" \
                          r"tokens such as \\=\\'auyhguywgerdbyubduiywebh' which is quite handy."
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(r"\\ssssssssssssssssssssss:\\ssssssssssssssss'auyhguywgerdbyubduiywebh'", match[0])
        self.assertEqual(r"\\=\\'auyhguywgerdbyubduiywebh'", match[1])

    def test_match_negative(self):
        """Test if negatives are recognized"""
        self.paste.body = "\\ssss:\\ssss'Aiughai'"
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session'\ssss:\\ssss'YTUF5GUY76ibuihIUIU98jJB+ÒÈÒà'"
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session_asd'\\aaa:\\ssss'auyhguywgerdbyubduiywebh'"
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = r"\"aws_session\"\\ssss:\ssss'auyhguywgerdbyubduiywebh'"
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = r"'aws_session'\\ssss$\\ssss'auyhguywgerdbyubduiywebh'"
        self.assertFalse(self.analyzer.match(self.paste))

        # We need to use triple strings here - https://stackoverflow.com/questions/27467870/escape-single-quote-in-raw-string-r
        self.paste.body = r"""Any text 'aws_session'\\ssss:\\ssss"auyhguywgerdbyubduiywebh" and more after"""
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest.mock import Mock, MagicMock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers import GenericAnalyzer


class TestGenericAnalyzer(unittest.TestCase):
    def setUp(self):
        self.mock_action = MagicMock(spec=BasicAction)

    def test_empty_match_func(self):
        """Check if a ValueError rises on empty match_func"""
        with self.assertRaises(ValueError, msg="No exception was raised although the match_func is None"):
            GenericAnalyzer(self.mock_action, None)

    def test_not_callable_match_func(self):
        """Check if ValueError rises on anything but a callable"""
        test_int = 1
        with self.assertRaises(ValueError, msg="No exception was raised although the match_func is not callable"):
            GenericAnalyzer(self.mock_action, test_int)

        test_string = "Test"
        with self.assertRaises(ValueError, msg="No exception was raised although the match_func is not callable"):
            GenericAnalyzer(self.mock_action, test_string)

    def test_perform(self):
        """Test to check if a given match_func will be executed when calling match()"""
        mock_func = Mock(return_value=True)
        mock_paste = Mock()

        ga = GenericAnalyzer(self.mock_action, mock_func)
        ga.match(paste=mock_paste)
        mock_func.assert_called_with(mock_paste)

    def test_perform_exception(self):
        """Test to check if the analyzer will catch and log exceptions from the match_func"""
        mock_exception = Mock(side_effect=ValueError)
        mock_exception.__name__ = "mock_match_exception"
        mock_paste = Mock()

        # Initialize analyzer and execute match function
        ga = GenericAnalyzer(self.mock_action, mock_exception)
        result = ga.match(mock_paste)

        # Make sure the analyzer method was called
        self.assertTrue(mock_exception.call_count > 0, msg="The mock has not been called!")

        # Make sure the analyzer returned false as result
        self.assertFalse(result, msg="The analyzer returned true although an exception was raised!")

    def test_empty_verify_func(self):
        """Test if passing None as verify function works out fine"""
        mock_func = Mock()
        ga = GenericAnalyzer(self.mock_action, mock_func, None)
        self.assertEqual(ga.verify_func, None)

    def test_not_callable_verify_func(self):
        """Check if ValueError rises on anything but a callable as verify_method (apart from None)"""
        mock_func = Mock()
        non_callable_obj = "This is a string, not a function you idiot!"
        with self.assertRaises(ValueError, msg="No exception was raised although the verify_func is not callable"):
            _ = GenericAnalyzer(self.mock_action, mock_func, non_callable_obj)

    def test_verify(self):
        """Test to check if a given verify_func will be executed when calling match()"""
        mock_match_func = Mock(return_value=True)
        mock_verify_func = Mock(return_value=True)
        mock_paste = Mock()

        ga = GenericAnalyzer(self.mock_action, match_func=mock_match_func, verify_func=mock_verify_func)
        ga.match(paste=mock_paste)
        mock_match_func.assert_called_with(mock_paste)

    def test_verify_negative(self):
        """Test to check if match() function returns False if verify retruns False"""
        match_ret_val = ["match1", "match2"]
        mock_match_func = Mock(return_value=match_ret_val)
        mock_verify_func = Mock(return_value=False)
        mock_paste = Mock()

        ga = GenericAnalyzer(self.mock_action, match_func=mock_match_func, verify_func=mock_verify_func)
        res = ga.match(paste=mock_paste)
        self.assertFalse(res, "The analyzer matched, although it shouldn't")
        mock_verify_func.assert_called_with(match_ret_val)

    def test_verify_exception(self):
        """Test to check if the analyzer will catch and log exceptions from the verify_func"""
        mock_match_func = Mock(return_value=True)
        mock_verify_exception = Mock(side_effect=ValueError)
        mock_verify_exception.__name__ = "mock_match_exception"
        mock_paste = Mock()

        # Initialize analyzer and execute match function
        ga = GenericAnalyzer(self.mock_action, match_func=mock_match_func, verify_func=mock_verify_exception)
        result = ga.match(mock_paste)

        # Make sure the verify method was called
        self.assertTrue(mock_verify_exception.call_count > 0, msg="The mock has not been called!")

        # Make sure the analyzer returned false as result
        self.assertFalse(result, msg="The analyzer returned true although an exception was raised!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.steamkeyanalyzer import SteamKeyAnalyzer


class TestSteamKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = SteamKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # steam key dump
        self.paste.body = "EZFNY-7GECB-94WRD"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "10NL7-7E8VK-ZUSOD"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "CB6AW-XHHB1-IANNO"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "E4XJ8-2MRI0-RX4I5"
        self.assertTrue(self.analyzer.match(self.paste))

        # 5 segment steam key
        self.paste.body = "1AB2C-D3FGH-456I7-JK8LM-NOP9Q"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        # part of a sentence
        self.paste.body = "Hey, I have your key right here: EL0SY-DC710-X0C5W!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("EL0SY-DC710-X0C5W", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        # Newline seperated steam key
        self.paste.body = "E4XJ8-2MRI0-RX4I5\nCIZ36-WD38P-QZI6U"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("E4XJ8-2MRI0-RX4I5", match[0])
        self.assertEqual("CIZ36-WD38P-QZI6U", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "CBCB6AW-XHHB1-IANNO"
        self.assertFalse(self.analyzer.match(self.paste))

        # Lower-case
        self.paste.body = "e4xj8-2mri0-rx4i5"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class OriginKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match Origin Keys"""

    def __init__(self, action):
        """
        Analyzer to match Origin Keys
        :param action: Single action or list of actions to be executed when a paste matches
        """
        # Tested Regex against https://pastebin.com/vyNANvwM
        regex = r"\b(?<!-)[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}\b(?!-)"

        super().__init__(action, regex)
